from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class MassMomentOfInertiaUnits(Enum):
        """
            MassMomentOfInertiaUnits enumeration
        """
        
        GramSquareMeter = 'GramSquareMeter'
        """
            
        """
        
        GramSquareDecimeter = 'GramSquareDecimeter'
        """
            
        """
        
        GramSquareCentimeter = 'GramSquareCentimeter'
        """
            
        """
        
        GramSquareMillimeter = 'GramSquareMillimeter'
        """
            
        """
        
        TonneSquareMeter = 'TonneSquareMeter'
        """
            
        """
        
        TonneSquareDecimeter = 'TonneSquareDecimeter'
        """
            
        """
        
        TonneSquareCentimeter = 'TonneSquareCentimeter'
        """
            
        """
        
        TonneSquareMilimeter = 'TonneSquareMilimeter'
        """
            
        """
        
        PoundSquareFoot = 'PoundSquareFoot'
        """
            
        """
        
        PoundSquareInch = 'PoundSquareInch'
        """
            
        """
        
        SlugSquareFoot = 'SlugSquareFoot'
        """
            
        """
        
        SlugSquareInch = 'SlugSquareInch'
        """
            
        """
        
        MilligramSquareMeter = 'MilligramSquareMeter'
        """
            
        """
        
        KilogramSquareMeter = 'KilogramSquareMeter'
        """
            
        """
        
        MilligramSquareDecimeter = 'MilligramSquareDecimeter'
        """
            
        """
        
        KilogramSquareDecimeter = 'KilogramSquareDecimeter'
        """
            
        """
        
        MilligramSquareCentimeter = 'MilligramSquareCentimeter'
        """
            
        """
        
        KilogramSquareCentimeter = 'KilogramSquareCentimeter'
        """
            
        """
        
        MilligramSquareMillimeter = 'MilligramSquareMillimeter'
        """
            
        """
        
        KilogramSquareMillimeter = 'KilogramSquareMillimeter'
        """
            
        """
        
        KilotonneSquareMeter = 'KilotonneSquareMeter'
        """
            
        """
        
        MegatonneSquareMeter = 'MegatonneSquareMeter'
        """
            
        """
        
        KilotonneSquareDecimeter = 'KilotonneSquareDecimeter'
        """
            
        """
        
        MegatonneSquareDecimeter = 'MegatonneSquareDecimeter'
        """
            
        """
        
        KilotonneSquareCentimeter = 'KilotonneSquareCentimeter'
        """
            
        """
        
        MegatonneSquareCentimeter = 'MegatonneSquareCentimeter'
        """
            
        """
        
        KilotonneSquareMilimeter = 'KilotonneSquareMilimeter'
        """
            
        """
        
        MegatonneSquareMilimeter = 'MegatonneSquareMilimeter'
        """
            
        """
        

class MassMomentOfInertiaDto:
    """
    A DTO representation of a MassMomentOfInertia

    Attributes:
        value (float): The value of the MassMomentOfInertia.
        unit (MassMomentOfInertiaUnits): The specific unit that the MassMomentOfInertia value is representing.
    """

    def __init__(self, value: float, unit: MassMomentOfInertiaUnits):
        """
        Create a new DTO representation of a MassMomentOfInertia

        Parameters:
            value (float): The value of the MassMomentOfInertia.
            unit (MassMomentOfInertiaUnits): The specific unit that the MassMomentOfInertia value is representing.
        """
        self.value: float = value
        """
        The value of the MassMomentOfInertia
        """
        self.unit: MassMomentOfInertiaUnits = unit
        """
        The specific unit that the MassMomentOfInertia value is representing
        """

    def to_json(self):
        """
        Get a MassMomentOfInertia DTO JSON object representing the current unit.

        :return: JSON object represents MassMomentOfInertia DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "KilogramSquareMeter"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of MassMomentOfInertia DTO from a json representation.

        :param data: The MassMomentOfInertia DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "KilogramSquareMeter"}
        :return: A new instance of MassMomentOfInertiaDto.
        :rtype: MassMomentOfInertiaDto
        """
        return MassMomentOfInertiaDto(value=data["value"], unit=MassMomentOfInertiaUnits(data["unit"]))


class MassMomentOfInertia(AbstractMeasure):
    """
    A property of body reflects how its mass is distributed with regard to an axis.

    Args:
        value (float): The value.
        from_unit (MassMomentOfInertiaUnits): The MassMomentOfInertia unit to create from, The default unit is KilogramSquareMeter
    """
    def __init__(self, value: float, from_unit: MassMomentOfInertiaUnits = MassMomentOfInertiaUnits.KilogramSquareMeter):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__gram_square_meters = None
        
        self.__gram_square_decimeters = None
        
        self.__gram_square_centimeters = None
        
        self.__gram_square_millimeters = None
        
        self.__tonne_square_meters = None
        
        self.__tonne_square_decimeters = None
        
        self.__tonne_square_centimeters = None
        
        self.__tonne_square_milimeters = None
        
        self.__pound_square_feet = None
        
        self.__pound_square_inches = None
        
        self.__slug_square_feet = None
        
        self.__slug_square_inches = None
        
        self.__milligram_square_meters = None
        
        self.__kilogram_square_meters = None
        
        self.__milligram_square_decimeters = None
        
        self.__kilogram_square_decimeters = None
        
        self.__milligram_square_centimeters = None
        
        self.__kilogram_square_centimeters = None
        
        self.__milligram_square_millimeters = None
        
        self.__kilogram_square_millimeters = None
        
        self.__kilotonne_square_meters = None
        
        self.__megatonne_square_meters = None
        
        self.__kilotonne_square_decimeters = None
        
        self.__megatonne_square_decimeters = None
        
        self.__kilotonne_square_centimeters = None
        
        self.__megatonne_square_centimeters = None
        
        self.__kilotonne_square_milimeters = None
        
        self.__megatonne_square_milimeters = None
        

    def convert(self, unit: MassMomentOfInertiaUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: MassMomentOfInertiaUnits = MassMomentOfInertiaUnits.KilogramSquareMeter) -> MassMomentOfInertiaDto:
        """
        Get a new instance of MassMomentOfInertia DTO representing the current unit.

        :param hold_in_unit: The specific MassMomentOfInertia unit to store the MassMomentOfInertia value in the DTO representation.
        :type hold_in_unit: MassMomentOfInertiaUnits
        :return: A new instance of MassMomentOfInertiaDto.
        :rtype: MassMomentOfInertiaDto
        """
        return MassMomentOfInertiaDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: MassMomentOfInertiaUnits = MassMomentOfInertiaUnits.KilogramSquareMeter):
        """
        Get a MassMomentOfInertia DTO JSON object representing the current unit.

        :param hold_in_unit: The specific MassMomentOfInertia unit to store the MassMomentOfInertia value in the DTO representation.
        :type hold_in_unit: MassMomentOfInertiaUnits
        :return: JSON object represents MassMomentOfInertia DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "KilogramSquareMeter"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(mass_moment_of_inertia_dto: MassMomentOfInertiaDto):
        """
        Obtain a new instance of MassMomentOfInertia from a DTO unit object.

        :param mass_moment_of_inertia_dto: The MassMomentOfInertia DTO representation.
        :type mass_moment_of_inertia_dto: MassMomentOfInertiaDto
        :return: A new instance of MassMomentOfInertia.
        :rtype: MassMomentOfInertia
        """
        return MassMomentOfInertia(mass_moment_of_inertia_dto.value, mass_moment_of_inertia_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of MassMomentOfInertia from a DTO unit json representation.

        :param data: The MassMomentOfInertia DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "KilogramSquareMeter"}
        :return: A new instance of MassMomentOfInertia.
        :rtype: MassMomentOfInertia
        """
        return MassMomentOfInertia.from_dto(MassMomentOfInertiaDto.from_json(data))

    def __convert_from_base(self, from_unit: MassMomentOfInertiaUnits) -> float:
        value = self._value
        
        if from_unit == MassMomentOfInertiaUnits.GramSquareMeter:
            return (value * 1e3)
        
        if from_unit == MassMomentOfInertiaUnits.GramSquareDecimeter:
            return (value * 1e5)
        
        if from_unit == MassMomentOfInertiaUnits.GramSquareCentimeter:
            return (value * 1e7)
        
        if from_unit == MassMomentOfInertiaUnits.GramSquareMillimeter:
            return (value * 1e9)
        
        if from_unit == MassMomentOfInertiaUnits.TonneSquareMeter:
            return (value * 1e-3)
        
        if from_unit == MassMomentOfInertiaUnits.TonneSquareDecimeter:
            return (value * 1e-1)
        
        if from_unit == MassMomentOfInertiaUnits.TonneSquareCentimeter:
            return (value * 1e1)
        
        if from_unit == MassMomentOfInertiaUnits.TonneSquareMilimeter:
            return (value * 1e3)
        
        if from_unit == MassMomentOfInertiaUnits.PoundSquareFoot:
            return (value / 4.21401101e-2)
        
        if from_unit == MassMomentOfInertiaUnits.PoundSquareInch:
            return (value / 2.9263965e-4)
        
        if from_unit == MassMomentOfInertiaUnits.SlugSquareFoot:
            return (value / 1.3558179619)
        
        if from_unit == MassMomentOfInertiaUnits.SlugSquareInch:
            return (value / 9.41540242e-3)
        
        if from_unit == MassMomentOfInertiaUnits.MilligramSquareMeter:
            return ((value * 1e3) / 0.001)
        
        if from_unit == MassMomentOfInertiaUnits.KilogramSquareMeter:
            return ((value * 1e3) / 1000.0)
        
        if from_unit == MassMomentOfInertiaUnits.MilligramSquareDecimeter:
            return ((value * 1e5) / 0.001)
        
        if from_unit == MassMomentOfInertiaUnits.KilogramSquareDecimeter:
            return ((value * 1e5) / 1000.0)
        
        if from_unit == MassMomentOfInertiaUnits.MilligramSquareCentimeter:
            return ((value * 1e7) / 0.001)
        
        if from_unit == MassMomentOfInertiaUnits.KilogramSquareCentimeter:
            return ((value * 1e7) / 1000.0)
        
        if from_unit == MassMomentOfInertiaUnits.MilligramSquareMillimeter:
            return ((value * 1e9) / 0.001)
        
        if from_unit == MassMomentOfInertiaUnits.KilogramSquareMillimeter:
            return ((value * 1e9) / 1000.0)
        
        if from_unit == MassMomentOfInertiaUnits.KilotonneSquareMeter:
            return ((value * 1e-3) / 1000.0)
        
        if from_unit == MassMomentOfInertiaUnits.MegatonneSquareMeter:
            return ((value * 1e-3) / 1000000.0)
        
        if from_unit == MassMomentOfInertiaUnits.KilotonneSquareDecimeter:
            return ((value * 1e-1) / 1000.0)
        
        if from_unit == MassMomentOfInertiaUnits.MegatonneSquareDecimeter:
            return ((value * 1e-1) / 1000000.0)
        
        if from_unit == MassMomentOfInertiaUnits.KilotonneSquareCentimeter:
            return ((value * 1e1) / 1000.0)
        
        if from_unit == MassMomentOfInertiaUnits.MegatonneSquareCentimeter:
            return ((value * 1e1) / 1000000.0)
        
        if from_unit == MassMomentOfInertiaUnits.KilotonneSquareMilimeter:
            return ((value * 1e3) / 1000.0)
        
        if from_unit == MassMomentOfInertiaUnits.MegatonneSquareMilimeter:
            return ((value * 1e3) / 1000000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: MassMomentOfInertiaUnits) -> float:
        
        if to_unit == MassMomentOfInertiaUnits.GramSquareMeter:
            return (value / 1e3)
        
        if to_unit == MassMomentOfInertiaUnits.GramSquareDecimeter:
            return (value / 1e5)
        
        if to_unit == MassMomentOfInertiaUnits.GramSquareCentimeter:
            return (value / 1e7)
        
        if to_unit == MassMomentOfInertiaUnits.GramSquareMillimeter:
            return (value / 1e9)
        
        if to_unit == MassMomentOfInertiaUnits.TonneSquareMeter:
            return (value / 1e-3)
        
        if to_unit == MassMomentOfInertiaUnits.TonneSquareDecimeter:
            return (value / 1e-1)
        
        if to_unit == MassMomentOfInertiaUnits.TonneSquareCentimeter:
            return (value / 1e1)
        
        if to_unit == MassMomentOfInertiaUnits.TonneSquareMilimeter:
            return (value / 1e3)
        
        if to_unit == MassMomentOfInertiaUnits.PoundSquareFoot:
            return (value * 4.21401101e-2)
        
        if to_unit == MassMomentOfInertiaUnits.PoundSquareInch:
            return (value * 2.9263965e-4)
        
        if to_unit == MassMomentOfInertiaUnits.SlugSquareFoot:
            return (value * 1.3558179619)
        
        if to_unit == MassMomentOfInertiaUnits.SlugSquareInch:
            return (value * 9.41540242e-3)
        
        if to_unit == MassMomentOfInertiaUnits.MilligramSquareMeter:
            return ((value / 1e3) * 0.001)
        
        if to_unit == MassMomentOfInertiaUnits.KilogramSquareMeter:
            return ((value / 1e3) * 1000.0)
        
        if to_unit == MassMomentOfInertiaUnits.MilligramSquareDecimeter:
            return ((value / 1e5) * 0.001)
        
        if to_unit == MassMomentOfInertiaUnits.KilogramSquareDecimeter:
            return ((value / 1e5) * 1000.0)
        
        if to_unit == MassMomentOfInertiaUnits.MilligramSquareCentimeter:
            return ((value / 1e7) * 0.001)
        
        if to_unit == MassMomentOfInertiaUnits.KilogramSquareCentimeter:
            return ((value / 1e7) * 1000.0)
        
        if to_unit == MassMomentOfInertiaUnits.MilligramSquareMillimeter:
            return ((value / 1e9) * 0.001)
        
        if to_unit == MassMomentOfInertiaUnits.KilogramSquareMillimeter:
            return ((value / 1e9) * 1000.0)
        
        if to_unit == MassMomentOfInertiaUnits.KilotonneSquareMeter:
            return ((value / 1e-3) * 1000.0)
        
        if to_unit == MassMomentOfInertiaUnits.MegatonneSquareMeter:
            return ((value / 1e-3) * 1000000.0)
        
        if to_unit == MassMomentOfInertiaUnits.KilotonneSquareDecimeter:
            return ((value / 1e-1) * 1000.0)
        
        if to_unit == MassMomentOfInertiaUnits.MegatonneSquareDecimeter:
            return ((value / 1e-1) * 1000000.0)
        
        if to_unit == MassMomentOfInertiaUnits.KilotonneSquareCentimeter:
            return ((value / 1e1) * 1000.0)
        
        if to_unit == MassMomentOfInertiaUnits.MegatonneSquareCentimeter:
            return ((value / 1e1) * 1000000.0)
        
        if to_unit == MassMomentOfInertiaUnits.KilotonneSquareMilimeter:
            return ((value / 1e3) * 1000.0)
        
        if to_unit == MassMomentOfInertiaUnits.MegatonneSquareMilimeter:
            return ((value / 1e3) * 1000000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_gram_square_meters(gram_square_meters: float):
        """
        Create a new instance of MassMomentOfInertia from a value in gram_square_meters.

        

        :param meters: The MassMomentOfInertia value in gram_square_meters.
        :type gram_square_meters: float
        :return: A new instance of MassMomentOfInertia.
        :rtype: MassMomentOfInertia
        """
        return MassMomentOfInertia(gram_square_meters, MassMomentOfInertiaUnits.GramSquareMeter)

    
    @staticmethod
    def from_gram_square_decimeters(gram_square_decimeters: float):
        """
        Create a new instance of MassMomentOfInertia from a value in gram_square_decimeters.

        

        :param meters: The MassMomentOfInertia value in gram_square_decimeters.
        :type gram_square_decimeters: float
        :return: A new instance of MassMomentOfInertia.
        :rtype: MassMomentOfInertia
        """
        return MassMomentOfInertia(gram_square_decimeters, MassMomentOfInertiaUnits.GramSquareDecimeter)

    
    @staticmethod
    def from_gram_square_centimeters(gram_square_centimeters: float):
        """
        Create a new instance of MassMomentOfInertia from a value in gram_square_centimeters.

        

        :param meters: The MassMomentOfInertia value in gram_square_centimeters.
        :type gram_square_centimeters: float
        :return: A new instance of MassMomentOfInertia.
        :rtype: MassMomentOfInertia
        """
        return MassMomentOfInertia(gram_square_centimeters, MassMomentOfInertiaUnits.GramSquareCentimeter)

    
    @staticmethod
    def from_gram_square_millimeters(gram_square_millimeters: float):
        """
        Create a new instance of MassMomentOfInertia from a value in gram_square_millimeters.

        

        :param meters: The MassMomentOfInertia value in gram_square_millimeters.
        :type gram_square_millimeters: float
        :return: A new instance of MassMomentOfInertia.
        :rtype: MassMomentOfInertia
        """
        return MassMomentOfInertia(gram_square_millimeters, MassMomentOfInertiaUnits.GramSquareMillimeter)

    
    @staticmethod
    def from_tonne_square_meters(tonne_square_meters: float):
        """
        Create a new instance of MassMomentOfInertia from a value in tonne_square_meters.

        

        :param meters: The MassMomentOfInertia value in tonne_square_meters.
        :type tonne_square_meters: float
        :return: A new instance of MassMomentOfInertia.
        :rtype: MassMomentOfInertia
        """
        return MassMomentOfInertia(tonne_square_meters, MassMomentOfInertiaUnits.TonneSquareMeter)

    
    @staticmethod
    def from_tonne_square_decimeters(tonne_square_decimeters: float):
        """
        Create a new instance of MassMomentOfInertia from a value in tonne_square_decimeters.

        

        :param meters: The MassMomentOfInertia value in tonne_square_decimeters.
        :type tonne_square_decimeters: float
        :return: A new instance of MassMomentOfInertia.
        :rtype: MassMomentOfInertia
        """
        return MassMomentOfInertia(tonne_square_decimeters, MassMomentOfInertiaUnits.TonneSquareDecimeter)

    
    @staticmethod
    def from_tonne_square_centimeters(tonne_square_centimeters: float):
        """
        Create a new instance of MassMomentOfInertia from a value in tonne_square_centimeters.

        

        :param meters: The MassMomentOfInertia value in tonne_square_centimeters.
        :type tonne_square_centimeters: float
        :return: A new instance of MassMomentOfInertia.
        :rtype: MassMomentOfInertia
        """
        return MassMomentOfInertia(tonne_square_centimeters, MassMomentOfInertiaUnits.TonneSquareCentimeter)

    
    @staticmethod
    def from_tonne_square_milimeters(tonne_square_milimeters: float):
        """
        Create a new instance of MassMomentOfInertia from a value in tonne_square_milimeters.

        

        :param meters: The MassMomentOfInertia value in tonne_square_milimeters.
        :type tonne_square_milimeters: float
        :return: A new instance of MassMomentOfInertia.
        :rtype: MassMomentOfInertia
        """
        return MassMomentOfInertia(tonne_square_milimeters, MassMomentOfInertiaUnits.TonneSquareMilimeter)

    
    @staticmethod
    def from_pound_square_feet(pound_square_feet: float):
        """
        Create a new instance of MassMomentOfInertia from a value in pound_square_feet.

        

        :param meters: The MassMomentOfInertia value in pound_square_feet.
        :type pound_square_feet: float
        :return: A new instance of MassMomentOfInertia.
        :rtype: MassMomentOfInertia
        """
        return MassMomentOfInertia(pound_square_feet, MassMomentOfInertiaUnits.PoundSquareFoot)

    
    @staticmethod
    def from_pound_square_inches(pound_square_inches: float):
        """
        Create a new instance of MassMomentOfInertia from a value in pound_square_inches.

        

        :param meters: The MassMomentOfInertia value in pound_square_inches.
        :type pound_square_inches: float
        :return: A new instance of MassMomentOfInertia.
        :rtype: MassMomentOfInertia
        """
        return MassMomentOfInertia(pound_square_inches, MassMomentOfInertiaUnits.PoundSquareInch)

    
    @staticmethod
    def from_slug_square_feet(slug_square_feet: float):
        """
        Create a new instance of MassMomentOfInertia from a value in slug_square_feet.

        

        :param meters: The MassMomentOfInertia value in slug_square_feet.
        :type slug_square_feet: float
        :return: A new instance of MassMomentOfInertia.
        :rtype: MassMomentOfInertia
        """
        return MassMomentOfInertia(slug_square_feet, MassMomentOfInertiaUnits.SlugSquareFoot)

    
    @staticmethod
    def from_slug_square_inches(slug_square_inches: float):
        """
        Create a new instance of MassMomentOfInertia from a value in slug_square_inches.

        

        :param meters: The MassMomentOfInertia value in slug_square_inches.
        :type slug_square_inches: float
        :return: A new instance of MassMomentOfInertia.
        :rtype: MassMomentOfInertia
        """
        return MassMomentOfInertia(slug_square_inches, MassMomentOfInertiaUnits.SlugSquareInch)

    
    @staticmethod
    def from_milligram_square_meters(milligram_square_meters: float):
        """
        Create a new instance of MassMomentOfInertia from a value in milligram_square_meters.

        

        :param meters: The MassMomentOfInertia value in milligram_square_meters.
        :type milligram_square_meters: float
        :return: A new instance of MassMomentOfInertia.
        :rtype: MassMomentOfInertia
        """
        return MassMomentOfInertia(milligram_square_meters, MassMomentOfInertiaUnits.MilligramSquareMeter)

    
    @staticmethod
    def from_kilogram_square_meters(kilogram_square_meters: float):
        """
        Create a new instance of MassMomentOfInertia from a value in kilogram_square_meters.

        

        :param meters: The MassMomentOfInertia value in kilogram_square_meters.
        :type kilogram_square_meters: float
        :return: A new instance of MassMomentOfInertia.
        :rtype: MassMomentOfInertia
        """
        return MassMomentOfInertia(kilogram_square_meters, MassMomentOfInertiaUnits.KilogramSquareMeter)

    
    @staticmethod
    def from_milligram_square_decimeters(milligram_square_decimeters: float):
        """
        Create a new instance of MassMomentOfInertia from a value in milligram_square_decimeters.

        

        :param meters: The MassMomentOfInertia value in milligram_square_decimeters.
        :type milligram_square_decimeters: float
        :return: A new instance of MassMomentOfInertia.
        :rtype: MassMomentOfInertia
        """
        return MassMomentOfInertia(milligram_square_decimeters, MassMomentOfInertiaUnits.MilligramSquareDecimeter)

    
    @staticmethod
    def from_kilogram_square_decimeters(kilogram_square_decimeters: float):
        """
        Create a new instance of MassMomentOfInertia from a value in kilogram_square_decimeters.

        

        :param meters: The MassMomentOfInertia value in kilogram_square_decimeters.
        :type kilogram_square_decimeters: float
        :return: A new instance of MassMomentOfInertia.
        :rtype: MassMomentOfInertia
        """
        return MassMomentOfInertia(kilogram_square_decimeters, MassMomentOfInertiaUnits.KilogramSquareDecimeter)

    
    @staticmethod
    def from_milligram_square_centimeters(milligram_square_centimeters: float):
        """
        Create a new instance of MassMomentOfInertia from a value in milligram_square_centimeters.

        

        :param meters: The MassMomentOfInertia value in milligram_square_centimeters.
        :type milligram_square_centimeters: float
        :return: A new instance of MassMomentOfInertia.
        :rtype: MassMomentOfInertia
        """
        return MassMomentOfInertia(milligram_square_centimeters, MassMomentOfInertiaUnits.MilligramSquareCentimeter)

    
    @staticmethod
    def from_kilogram_square_centimeters(kilogram_square_centimeters: float):
        """
        Create a new instance of MassMomentOfInertia from a value in kilogram_square_centimeters.

        

        :param meters: The MassMomentOfInertia value in kilogram_square_centimeters.
        :type kilogram_square_centimeters: float
        :return: A new instance of MassMomentOfInertia.
        :rtype: MassMomentOfInertia
        """
        return MassMomentOfInertia(kilogram_square_centimeters, MassMomentOfInertiaUnits.KilogramSquareCentimeter)

    
    @staticmethod
    def from_milligram_square_millimeters(milligram_square_millimeters: float):
        """
        Create a new instance of MassMomentOfInertia from a value in milligram_square_millimeters.

        

        :param meters: The MassMomentOfInertia value in milligram_square_millimeters.
        :type milligram_square_millimeters: float
        :return: A new instance of MassMomentOfInertia.
        :rtype: MassMomentOfInertia
        """
        return MassMomentOfInertia(milligram_square_millimeters, MassMomentOfInertiaUnits.MilligramSquareMillimeter)

    
    @staticmethod
    def from_kilogram_square_millimeters(kilogram_square_millimeters: float):
        """
        Create a new instance of MassMomentOfInertia from a value in kilogram_square_millimeters.

        

        :param meters: The MassMomentOfInertia value in kilogram_square_millimeters.
        :type kilogram_square_millimeters: float
        :return: A new instance of MassMomentOfInertia.
        :rtype: MassMomentOfInertia
        """
        return MassMomentOfInertia(kilogram_square_millimeters, MassMomentOfInertiaUnits.KilogramSquareMillimeter)

    
    @staticmethod
    def from_kilotonne_square_meters(kilotonne_square_meters: float):
        """
        Create a new instance of MassMomentOfInertia from a value in kilotonne_square_meters.

        

        :param meters: The MassMomentOfInertia value in kilotonne_square_meters.
        :type kilotonne_square_meters: float
        :return: A new instance of MassMomentOfInertia.
        :rtype: MassMomentOfInertia
        """
        return MassMomentOfInertia(kilotonne_square_meters, MassMomentOfInertiaUnits.KilotonneSquareMeter)

    
    @staticmethod
    def from_megatonne_square_meters(megatonne_square_meters: float):
        """
        Create a new instance of MassMomentOfInertia from a value in megatonne_square_meters.

        

        :param meters: The MassMomentOfInertia value in megatonne_square_meters.
        :type megatonne_square_meters: float
        :return: A new instance of MassMomentOfInertia.
        :rtype: MassMomentOfInertia
        """
        return MassMomentOfInertia(megatonne_square_meters, MassMomentOfInertiaUnits.MegatonneSquareMeter)

    
    @staticmethod
    def from_kilotonne_square_decimeters(kilotonne_square_decimeters: float):
        """
        Create a new instance of MassMomentOfInertia from a value in kilotonne_square_decimeters.

        

        :param meters: The MassMomentOfInertia value in kilotonne_square_decimeters.
        :type kilotonne_square_decimeters: float
        :return: A new instance of MassMomentOfInertia.
        :rtype: MassMomentOfInertia
        """
        return MassMomentOfInertia(kilotonne_square_decimeters, MassMomentOfInertiaUnits.KilotonneSquareDecimeter)

    
    @staticmethod
    def from_megatonne_square_decimeters(megatonne_square_decimeters: float):
        """
        Create a new instance of MassMomentOfInertia from a value in megatonne_square_decimeters.

        

        :param meters: The MassMomentOfInertia value in megatonne_square_decimeters.
        :type megatonne_square_decimeters: float
        :return: A new instance of MassMomentOfInertia.
        :rtype: MassMomentOfInertia
        """
        return MassMomentOfInertia(megatonne_square_decimeters, MassMomentOfInertiaUnits.MegatonneSquareDecimeter)

    
    @staticmethod
    def from_kilotonne_square_centimeters(kilotonne_square_centimeters: float):
        """
        Create a new instance of MassMomentOfInertia from a value in kilotonne_square_centimeters.

        

        :param meters: The MassMomentOfInertia value in kilotonne_square_centimeters.
        :type kilotonne_square_centimeters: float
        :return: A new instance of MassMomentOfInertia.
        :rtype: MassMomentOfInertia
        """
        return MassMomentOfInertia(kilotonne_square_centimeters, MassMomentOfInertiaUnits.KilotonneSquareCentimeter)

    
    @staticmethod
    def from_megatonne_square_centimeters(megatonne_square_centimeters: float):
        """
        Create a new instance of MassMomentOfInertia from a value in megatonne_square_centimeters.

        

        :param meters: The MassMomentOfInertia value in megatonne_square_centimeters.
        :type megatonne_square_centimeters: float
        :return: A new instance of MassMomentOfInertia.
        :rtype: MassMomentOfInertia
        """
        return MassMomentOfInertia(megatonne_square_centimeters, MassMomentOfInertiaUnits.MegatonneSquareCentimeter)

    
    @staticmethod
    def from_kilotonne_square_milimeters(kilotonne_square_milimeters: float):
        """
        Create a new instance of MassMomentOfInertia from a value in kilotonne_square_milimeters.

        

        :param meters: The MassMomentOfInertia value in kilotonne_square_milimeters.
        :type kilotonne_square_milimeters: float
        :return: A new instance of MassMomentOfInertia.
        :rtype: MassMomentOfInertia
        """
        return MassMomentOfInertia(kilotonne_square_milimeters, MassMomentOfInertiaUnits.KilotonneSquareMilimeter)

    
    @staticmethod
    def from_megatonne_square_milimeters(megatonne_square_milimeters: float):
        """
        Create a new instance of MassMomentOfInertia from a value in megatonne_square_milimeters.

        

        :param meters: The MassMomentOfInertia value in megatonne_square_milimeters.
        :type megatonne_square_milimeters: float
        :return: A new instance of MassMomentOfInertia.
        :rtype: MassMomentOfInertia
        """
        return MassMomentOfInertia(megatonne_square_milimeters, MassMomentOfInertiaUnits.MegatonneSquareMilimeter)

    
    @property
    def gram_square_meters(self) -> float:
        """
        
        """
        if self.__gram_square_meters != None:
            return self.__gram_square_meters
        self.__gram_square_meters = self.__convert_from_base(MassMomentOfInertiaUnits.GramSquareMeter)
        return self.__gram_square_meters

    
    @property
    def gram_square_decimeters(self) -> float:
        """
        
        """
        if self.__gram_square_decimeters != None:
            return self.__gram_square_decimeters
        self.__gram_square_decimeters = self.__convert_from_base(MassMomentOfInertiaUnits.GramSquareDecimeter)
        return self.__gram_square_decimeters

    
    @property
    def gram_square_centimeters(self) -> float:
        """
        
        """
        if self.__gram_square_centimeters != None:
            return self.__gram_square_centimeters
        self.__gram_square_centimeters = self.__convert_from_base(MassMomentOfInertiaUnits.GramSquareCentimeter)
        return self.__gram_square_centimeters

    
    @property
    def gram_square_millimeters(self) -> float:
        """
        
        """
        if self.__gram_square_millimeters != None:
            return self.__gram_square_millimeters
        self.__gram_square_millimeters = self.__convert_from_base(MassMomentOfInertiaUnits.GramSquareMillimeter)
        return self.__gram_square_millimeters

    
    @property
    def tonne_square_meters(self) -> float:
        """
        
        """
        if self.__tonne_square_meters != None:
            return self.__tonne_square_meters
        self.__tonne_square_meters = self.__convert_from_base(MassMomentOfInertiaUnits.TonneSquareMeter)
        return self.__tonne_square_meters

    
    @property
    def tonne_square_decimeters(self) -> float:
        """
        
        """
        if self.__tonne_square_decimeters != None:
            return self.__tonne_square_decimeters
        self.__tonne_square_decimeters = self.__convert_from_base(MassMomentOfInertiaUnits.TonneSquareDecimeter)
        return self.__tonne_square_decimeters

    
    @property
    def tonne_square_centimeters(self) -> float:
        """
        
        """
        if self.__tonne_square_centimeters != None:
            return self.__tonne_square_centimeters
        self.__tonne_square_centimeters = self.__convert_from_base(MassMomentOfInertiaUnits.TonneSquareCentimeter)
        return self.__tonne_square_centimeters

    
    @property
    def tonne_square_milimeters(self) -> float:
        """
        
        """
        if self.__tonne_square_milimeters != None:
            return self.__tonne_square_milimeters
        self.__tonne_square_milimeters = self.__convert_from_base(MassMomentOfInertiaUnits.TonneSquareMilimeter)
        return self.__tonne_square_milimeters

    
    @property
    def pound_square_feet(self) -> float:
        """
        
        """
        if self.__pound_square_feet != None:
            return self.__pound_square_feet
        self.__pound_square_feet = self.__convert_from_base(MassMomentOfInertiaUnits.PoundSquareFoot)
        return self.__pound_square_feet

    
    @property
    def pound_square_inches(self) -> float:
        """
        
        """
        if self.__pound_square_inches != None:
            return self.__pound_square_inches
        self.__pound_square_inches = self.__convert_from_base(MassMomentOfInertiaUnits.PoundSquareInch)
        return self.__pound_square_inches

    
    @property
    def slug_square_feet(self) -> float:
        """
        
        """
        if self.__slug_square_feet != None:
            return self.__slug_square_feet
        self.__slug_square_feet = self.__convert_from_base(MassMomentOfInertiaUnits.SlugSquareFoot)
        return self.__slug_square_feet

    
    @property
    def slug_square_inches(self) -> float:
        """
        
        """
        if self.__slug_square_inches != None:
            return self.__slug_square_inches
        self.__slug_square_inches = self.__convert_from_base(MassMomentOfInertiaUnits.SlugSquareInch)
        return self.__slug_square_inches

    
    @property
    def milligram_square_meters(self) -> float:
        """
        
        """
        if self.__milligram_square_meters != None:
            return self.__milligram_square_meters
        self.__milligram_square_meters = self.__convert_from_base(MassMomentOfInertiaUnits.MilligramSquareMeter)
        return self.__milligram_square_meters

    
    @property
    def kilogram_square_meters(self) -> float:
        """
        
        """
        if self.__kilogram_square_meters != None:
            return self.__kilogram_square_meters
        self.__kilogram_square_meters = self.__convert_from_base(MassMomentOfInertiaUnits.KilogramSquareMeter)
        return self.__kilogram_square_meters

    
    @property
    def milligram_square_decimeters(self) -> float:
        """
        
        """
        if self.__milligram_square_decimeters != None:
            return self.__milligram_square_decimeters
        self.__milligram_square_decimeters = self.__convert_from_base(MassMomentOfInertiaUnits.MilligramSquareDecimeter)
        return self.__milligram_square_decimeters

    
    @property
    def kilogram_square_decimeters(self) -> float:
        """
        
        """
        if self.__kilogram_square_decimeters != None:
            return self.__kilogram_square_decimeters
        self.__kilogram_square_decimeters = self.__convert_from_base(MassMomentOfInertiaUnits.KilogramSquareDecimeter)
        return self.__kilogram_square_decimeters

    
    @property
    def milligram_square_centimeters(self) -> float:
        """
        
        """
        if self.__milligram_square_centimeters != None:
            return self.__milligram_square_centimeters
        self.__milligram_square_centimeters = self.__convert_from_base(MassMomentOfInertiaUnits.MilligramSquareCentimeter)
        return self.__milligram_square_centimeters

    
    @property
    def kilogram_square_centimeters(self) -> float:
        """
        
        """
        if self.__kilogram_square_centimeters != None:
            return self.__kilogram_square_centimeters
        self.__kilogram_square_centimeters = self.__convert_from_base(MassMomentOfInertiaUnits.KilogramSquareCentimeter)
        return self.__kilogram_square_centimeters

    
    @property
    def milligram_square_millimeters(self) -> float:
        """
        
        """
        if self.__milligram_square_millimeters != None:
            return self.__milligram_square_millimeters
        self.__milligram_square_millimeters = self.__convert_from_base(MassMomentOfInertiaUnits.MilligramSquareMillimeter)
        return self.__milligram_square_millimeters

    
    @property
    def kilogram_square_millimeters(self) -> float:
        """
        
        """
        if self.__kilogram_square_millimeters != None:
            return self.__kilogram_square_millimeters
        self.__kilogram_square_millimeters = self.__convert_from_base(MassMomentOfInertiaUnits.KilogramSquareMillimeter)
        return self.__kilogram_square_millimeters

    
    @property
    def kilotonne_square_meters(self) -> float:
        """
        
        """
        if self.__kilotonne_square_meters != None:
            return self.__kilotonne_square_meters
        self.__kilotonne_square_meters = self.__convert_from_base(MassMomentOfInertiaUnits.KilotonneSquareMeter)
        return self.__kilotonne_square_meters

    
    @property
    def megatonne_square_meters(self) -> float:
        """
        
        """
        if self.__megatonne_square_meters != None:
            return self.__megatonne_square_meters
        self.__megatonne_square_meters = self.__convert_from_base(MassMomentOfInertiaUnits.MegatonneSquareMeter)
        return self.__megatonne_square_meters

    
    @property
    def kilotonne_square_decimeters(self) -> float:
        """
        
        """
        if self.__kilotonne_square_decimeters != None:
            return self.__kilotonne_square_decimeters
        self.__kilotonne_square_decimeters = self.__convert_from_base(MassMomentOfInertiaUnits.KilotonneSquareDecimeter)
        return self.__kilotonne_square_decimeters

    
    @property
    def megatonne_square_decimeters(self) -> float:
        """
        
        """
        if self.__megatonne_square_decimeters != None:
            return self.__megatonne_square_decimeters
        self.__megatonne_square_decimeters = self.__convert_from_base(MassMomentOfInertiaUnits.MegatonneSquareDecimeter)
        return self.__megatonne_square_decimeters

    
    @property
    def kilotonne_square_centimeters(self) -> float:
        """
        
        """
        if self.__kilotonne_square_centimeters != None:
            return self.__kilotonne_square_centimeters
        self.__kilotonne_square_centimeters = self.__convert_from_base(MassMomentOfInertiaUnits.KilotonneSquareCentimeter)
        return self.__kilotonne_square_centimeters

    
    @property
    def megatonne_square_centimeters(self) -> float:
        """
        
        """
        if self.__megatonne_square_centimeters != None:
            return self.__megatonne_square_centimeters
        self.__megatonne_square_centimeters = self.__convert_from_base(MassMomentOfInertiaUnits.MegatonneSquareCentimeter)
        return self.__megatonne_square_centimeters

    
    @property
    def kilotonne_square_milimeters(self) -> float:
        """
        
        """
        if self.__kilotonne_square_milimeters != None:
            return self.__kilotonne_square_milimeters
        self.__kilotonne_square_milimeters = self.__convert_from_base(MassMomentOfInertiaUnits.KilotonneSquareMilimeter)
        return self.__kilotonne_square_milimeters

    
    @property
    def megatonne_square_milimeters(self) -> float:
        """
        
        """
        if self.__megatonne_square_milimeters != None:
            return self.__megatonne_square_milimeters
        self.__megatonne_square_milimeters = self.__convert_from_base(MassMomentOfInertiaUnits.MegatonneSquareMilimeter)
        return self.__megatonne_square_milimeters

    
    def to_string(self, unit: MassMomentOfInertiaUnits = MassMomentOfInertiaUnits.KilogramSquareMeter, fractional_digits: int = None) -> str:
        """
        Format the MassMomentOfInertia to a string.
        
        Note: the default format for MassMomentOfInertia is KilogramSquareMeter.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the MassMomentOfInertia. Default is 'KilogramSquareMeter'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == MassMomentOfInertiaUnits.GramSquareMeter:
            return f"""{super()._truncate_fraction_digits(self.gram_square_meters, fractional_digits)} g·m²"""
        
        if unit == MassMomentOfInertiaUnits.GramSquareDecimeter:
            return f"""{super()._truncate_fraction_digits(self.gram_square_decimeters, fractional_digits)} g·dm²"""
        
        if unit == MassMomentOfInertiaUnits.GramSquareCentimeter:
            return f"""{super()._truncate_fraction_digits(self.gram_square_centimeters, fractional_digits)} g·cm²"""
        
        if unit == MassMomentOfInertiaUnits.GramSquareMillimeter:
            return f"""{super()._truncate_fraction_digits(self.gram_square_millimeters, fractional_digits)} g·mm²"""
        
        if unit == MassMomentOfInertiaUnits.TonneSquareMeter:
            return f"""{super()._truncate_fraction_digits(self.tonne_square_meters, fractional_digits)} t·m²"""
        
        if unit == MassMomentOfInertiaUnits.TonneSquareDecimeter:
            return f"""{super()._truncate_fraction_digits(self.tonne_square_decimeters, fractional_digits)} t·dm²"""
        
        if unit == MassMomentOfInertiaUnits.TonneSquareCentimeter:
            return f"""{super()._truncate_fraction_digits(self.tonne_square_centimeters, fractional_digits)} t·cm²"""
        
        if unit == MassMomentOfInertiaUnits.TonneSquareMilimeter:
            return f"""{super()._truncate_fraction_digits(self.tonne_square_milimeters, fractional_digits)} t·mm²"""
        
        if unit == MassMomentOfInertiaUnits.PoundSquareFoot:
            return f"""{super()._truncate_fraction_digits(self.pound_square_feet, fractional_digits)} lb·ft²"""
        
        if unit == MassMomentOfInertiaUnits.PoundSquareInch:
            return f"""{super()._truncate_fraction_digits(self.pound_square_inches, fractional_digits)} lb·in²"""
        
        if unit == MassMomentOfInertiaUnits.SlugSquareFoot:
            return f"""{super()._truncate_fraction_digits(self.slug_square_feet, fractional_digits)} slug·ft²"""
        
        if unit == MassMomentOfInertiaUnits.SlugSquareInch:
            return f"""{super()._truncate_fraction_digits(self.slug_square_inches, fractional_digits)} slug·in²"""
        
        if unit == MassMomentOfInertiaUnits.MilligramSquareMeter:
            return f"""{super()._truncate_fraction_digits(self.milligram_square_meters, fractional_digits)} mg·m²"""
        
        if unit == MassMomentOfInertiaUnits.KilogramSquareMeter:
            return f"""{super()._truncate_fraction_digits(self.kilogram_square_meters, fractional_digits)} kg·m²"""
        
        if unit == MassMomentOfInertiaUnits.MilligramSquareDecimeter:
            return f"""{super()._truncate_fraction_digits(self.milligram_square_decimeters, fractional_digits)} mg·dm²"""
        
        if unit == MassMomentOfInertiaUnits.KilogramSquareDecimeter:
            return f"""{super()._truncate_fraction_digits(self.kilogram_square_decimeters, fractional_digits)} kg·dm²"""
        
        if unit == MassMomentOfInertiaUnits.MilligramSquareCentimeter:
            return f"""{super()._truncate_fraction_digits(self.milligram_square_centimeters, fractional_digits)} mg·cm²"""
        
        if unit == MassMomentOfInertiaUnits.KilogramSquareCentimeter:
            return f"""{super()._truncate_fraction_digits(self.kilogram_square_centimeters, fractional_digits)} kg·cm²"""
        
        if unit == MassMomentOfInertiaUnits.MilligramSquareMillimeter:
            return f"""{super()._truncate_fraction_digits(self.milligram_square_millimeters, fractional_digits)} mg·mm²"""
        
        if unit == MassMomentOfInertiaUnits.KilogramSquareMillimeter:
            return f"""{super()._truncate_fraction_digits(self.kilogram_square_millimeters, fractional_digits)} kg·mm²"""
        
        if unit == MassMomentOfInertiaUnits.KilotonneSquareMeter:
            return f"""{super()._truncate_fraction_digits(self.kilotonne_square_meters, fractional_digits)} kt·m²"""
        
        if unit == MassMomentOfInertiaUnits.MegatonneSquareMeter:
            return f"""{super()._truncate_fraction_digits(self.megatonne_square_meters, fractional_digits)} Mt·m²"""
        
        if unit == MassMomentOfInertiaUnits.KilotonneSquareDecimeter:
            return f"""{super()._truncate_fraction_digits(self.kilotonne_square_decimeters, fractional_digits)} kt·dm²"""
        
        if unit == MassMomentOfInertiaUnits.MegatonneSquareDecimeter:
            return f"""{super()._truncate_fraction_digits(self.megatonne_square_decimeters, fractional_digits)} Mt·dm²"""
        
        if unit == MassMomentOfInertiaUnits.KilotonneSquareCentimeter:
            return f"""{super()._truncate_fraction_digits(self.kilotonne_square_centimeters, fractional_digits)} kt·cm²"""
        
        if unit == MassMomentOfInertiaUnits.MegatonneSquareCentimeter:
            return f"""{super()._truncate_fraction_digits(self.megatonne_square_centimeters, fractional_digits)} Mt·cm²"""
        
        if unit == MassMomentOfInertiaUnits.KilotonneSquareMilimeter:
            return f"""{super()._truncate_fraction_digits(self.kilotonne_square_milimeters, fractional_digits)} kt·mm²"""
        
        if unit == MassMomentOfInertiaUnits.MegatonneSquareMilimeter:
            return f"""{super()._truncate_fraction_digits(self.megatonne_square_milimeters, fractional_digits)} Mt·mm²"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: MassMomentOfInertiaUnits = MassMomentOfInertiaUnits.KilogramSquareMeter) -> str:
        """
        Get MassMomentOfInertia unit abbreviation.
        Note! the default abbreviation for MassMomentOfInertia is KilogramSquareMeter.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == MassMomentOfInertiaUnits.GramSquareMeter:
            return """g·m²"""
        
        if unit_abbreviation == MassMomentOfInertiaUnits.GramSquareDecimeter:
            return """g·dm²"""
        
        if unit_abbreviation == MassMomentOfInertiaUnits.GramSquareCentimeter:
            return """g·cm²"""
        
        if unit_abbreviation == MassMomentOfInertiaUnits.GramSquareMillimeter:
            return """g·mm²"""
        
        if unit_abbreviation == MassMomentOfInertiaUnits.TonneSquareMeter:
            return """t·m²"""
        
        if unit_abbreviation == MassMomentOfInertiaUnits.TonneSquareDecimeter:
            return """t·dm²"""
        
        if unit_abbreviation == MassMomentOfInertiaUnits.TonneSquareCentimeter:
            return """t·cm²"""
        
        if unit_abbreviation == MassMomentOfInertiaUnits.TonneSquareMilimeter:
            return """t·mm²"""
        
        if unit_abbreviation == MassMomentOfInertiaUnits.PoundSquareFoot:
            return """lb·ft²"""
        
        if unit_abbreviation == MassMomentOfInertiaUnits.PoundSquareInch:
            return """lb·in²"""
        
        if unit_abbreviation == MassMomentOfInertiaUnits.SlugSquareFoot:
            return """slug·ft²"""
        
        if unit_abbreviation == MassMomentOfInertiaUnits.SlugSquareInch:
            return """slug·in²"""
        
        if unit_abbreviation == MassMomentOfInertiaUnits.MilligramSquareMeter:
            return """mg·m²"""
        
        if unit_abbreviation == MassMomentOfInertiaUnits.KilogramSquareMeter:
            return """kg·m²"""
        
        if unit_abbreviation == MassMomentOfInertiaUnits.MilligramSquareDecimeter:
            return """mg·dm²"""
        
        if unit_abbreviation == MassMomentOfInertiaUnits.KilogramSquareDecimeter:
            return """kg·dm²"""
        
        if unit_abbreviation == MassMomentOfInertiaUnits.MilligramSquareCentimeter:
            return """mg·cm²"""
        
        if unit_abbreviation == MassMomentOfInertiaUnits.KilogramSquareCentimeter:
            return """kg·cm²"""
        
        if unit_abbreviation == MassMomentOfInertiaUnits.MilligramSquareMillimeter:
            return """mg·mm²"""
        
        if unit_abbreviation == MassMomentOfInertiaUnits.KilogramSquareMillimeter:
            return """kg·mm²"""
        
        if unit_abbreviation == MassMomentOfInertiaUnits.KilotonneSquareMeter:
            return """kt·m²"""
        
        if unit_abbreviation == MassMomentOfInertiaUnits.MegatonneSquareMeter:
            return """Mt·m²"""
        
        if unit_abbreviation == MassMomentOfInertiaUnits.KilotonneSquareDecimeter:
            return """kt·dm²"""
        
        if unit_abbreviation == MassMomentOfInertiaUnits.MegatonneSquareDecimeter:
            return """Mt·dm²"""
        
        if unit_abbreviation == MassMomentOfInertiaUnits.KilotonneSquareCentimeter:
            return """kt·cm²"""
        
        if unit_abbreviation == MassMomentOfInertiaUnits.MegatonneSquareCentimeter:
            return """Mt·cm²"""
        
        if unit_abbreviation == MassMomentOfInertiaUnits.KilotonneSquareMilimeter:
            return """kt·mm²"""
        
        if unit_abbreviation == MassMomentOfInertiaUnits.MegatonneSquareMilimeter:
            return """Mt·mm²"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class TemperatureChangeRateUnits(Enum):
        """
            TemperatureChangeRateUnits enumeration
        """
        
        DegreeCelsiusPerSecond = 'DegreeCelsiusPerSecond'
        """
            
        """
        
        DegreeCelsiusPerMinute = 'DegreeCelsiusPerMinute'
        """
            
        """
        
        NanodegreeCelsiusPerSecond = 'NanodegreeCelsiusPerSecond'
        """
            
        """
        
        MicrodegreeCelsiusPerSecond = 'MicrodegreeCelsiusPerSecond'
        """
            
        """
        
        MillidegreeCelsiusPerSecond = 'MillidegreeCelsiusPerSecond'
        """
            
        """
        
        CentidegreeCelsiusPerSecond = 'CentidegreeCelsiusPerSecond'
        """
            
        """
        
        DecidegreeCelsiusPerSecond = 'DecidegreeCelsiusPerSecond'
        """
            
        """
        
        DecadegreeCelsiusPerSecond = 'DecadegreeCelsiusPerSecond'
        """
            
        """
        
        HectodegreeCelsiusPerSecond = 'HectodegreeCelsiusPerSecond'
        """
            
        """
        
        KilodegreeCelsiusPerSecond = 'KilodegreeCelsiusPerSecond'
        """
            
        """
        

class TemperatureChangeRateDto:
    """
    A DTO representation of a TemperatureChangeRate

    Attributes:
        value (float): The value of the TemperatureChangeRate.
        unit (TemperatureChangeRateUnits): The specific unit that the TemperatureChangeRate value is representing.
    """

    def __init__(self, value: float, unit: TemperatureChangeRateUnits):
        """
        Create a new DTO representation of a TemperatureChangeRate

        Parameters:
            value (float): The value of the TemperatureChangeRate.
            unit (TemperatureChangeRateUnits): The specific unit that the TemperatureChangeRate value is representing.
        """
        self.value: float = value
        """
        The value of the TemperatureChangeRate
        """
        self.unit: TemperatureChangeRateUnits = unit
        """
        The specific unit that the TemperatureChangeRate value is representing
        """

    def to_json(self):
        """
        Get a TemperatureChangeRate DTO JSON object representing the current unit.

        :return: JSON object represents TemperatureChangeRate DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "DegreeCelsiusPerSecond"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of TemperatureChangeRate DTO from a json representation.

        :param data: The TemperatureChangeRate DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "DegreeCelsiusPerSecond"}
        :return: A new instance of TemperatureChangeRateDto.
        :rtype: TemperatureChangeRateDto
        """
        return TemperatureChangeRateDto(value=data["value"], unit=TemperatureChangeRateUnits(data["unit"]))


class TemperatureChangeRate(AbstractMeasure):
    """
    Temperature change rate is the ratio of the temperature change to the time during which the change occurred (value of temperature changes per unit time).

    Args:
        value (float): The value.
        from_unit (TemperatureChangeRateUnits): The TemperatureChangeRate unit to create from, The default unit is DegreeCelsiusPerSecond
    """
    def __init__(self, value: float, from_unit: TemperatureChangeRateUnits = TemperatureChangeRateUnits.DegreeCelsiusPerSecond):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__degrees_celsius_per_second = None
        
        self.__degrees_celsius_per_minute = None
        
        self.__nanodegrees_celsius_per_second = None
        
        self.__microdegrees_celsius_per_second = None
        
        self.__millidegrees_celsius_per_second = None
        
        self.__centidegrees_celsius_per_second = None
        
        self.__decidegrees_celsius_per_second = None
        
        self.__decadegrees_celsius_per_second = None
        
        self.__hectodegrees_celsius_per_second = None
        
        self.__kilodegrees_celsius_per_second = None
        

    def convert(self, unit: TemperatureChangeRateUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: TemperatureChangeRateUnits = TemperatureChangeRateUnits.DegreeCelsiusPerSecond) -> TemperatureChangeRateDto:
        """
        Get a new instance of TemperatureChangeRate DTO representing the current unit.

        :param hold_in_unit: The specific TemperatureChangeRate unit to store the TemperatureChangeRate value in the DTO representation.
        :type hold_in_unit: TemperatureChangeRateUnits
        :return: A new instance of TemperatureChangeRateDto.
        :rtype: TemperatureChangeRateDto
        """
        return TemperatureChangeRateDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: TemperatureChangeRateUnits = TemperatureChangeRateUnits.DegreeCelsiusPerSecond):
        """
        Get a TemperatureChangeRate DTO JSON object representing the current unit.

        :param hold_in_unit: The specific TemperatureChangeRate unit to store the TemperatureChangeRate value in the DTO representation.
        :type hold_in_unit: TemperatureChangeRateUnits
        :return: JSON object represents TemperatureChangeRate DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "DegreeCelsiusPerSecond"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(temperature_change_rate_dto: TemperatureChangeRateDto):
        """
        Obtain a new instance of TemperatureChangeRate from a DTO unit object.

        :param temperature_change_rate_dto: The TemperatureChangeRate DTO representation.
        :type temperature_change_rate_dto: TemperatureChangeRateDto
        :return: A new instance of TemperatureChangeRate.
        :rtype: TemperatureChangeRate
        """
        return TemperatureChangeRate(temperature_change_rate_dto.value, temperature_change_rate_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of TemperatureChangeRate from a DTO unit json representation.

        :param data: The TemperatureChangeRate DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "DegreeCelsiusPerSecond"}
        :return: A new instance of TemperatureChangeRate.
        :rtype: TemperatureChangeRate
        """
        return TemperatureChangeRate.from_dto(TemperatureChangeRateDto.from_json(data))

    def __convert_from_base(self, from_unit: TemperatureChangeRateUnits) -> float:
        value = self._value
        
        if from_unit == TemperatureChangeRateUnits.DegreeCelsiusPerSecond:
            return (value)
        
        if from_unit == TemperatureChangeRateUnits.DegreeCelsiusPerMinute:
            return (value * 60)
        
        if from_unit == TemperatureChangeRateUnits.NanodegreeCelsiusPerSecond:
            return ((value) / 1e-09)
        
        if from_unit == TemperatureChangeRateUnits.MicrodegreeCelsiusPerSecond:
            return ((value) / 1e-06)
        
        if from_unit == TemperatureChangeRateUnits.MillidegreeCelsiusPerSecond:
            return ((value) / 0.001)
        
        if from_unit == TemperatureChangeRateUnits.CentidegreeCelsiusPerSecond:
            return ((value) / 0.01)
        
        if from_unit == TemperatureChangeRateUnits.DecidegreeCelsiusPerSecond:
            return ((value) / 0.1)
        
        if from_unit == TemperatureChangeRateUnits.DecadegreeCelsiusPerSecond:
            return ((value) / 10.0)
        
        if from_unit == TemperatureChangeRateUnits.HectodegreeCelsiusPerSecond:
            return ((value) / 100.0)
        
        if from_unit == TemperatureChangeRateUnits.KilodegreeCelsiusPerSecond:
            return ((value) / 1000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: TemperatureChangeRateUnits) -> float:
        
        if to_unit == TemperatureChangeRateUnits.DegreeCelsiusPerSecond:
            return (value)
        
        if to_unit == TemperatureChangeRateUnits.DegreeCelsiusPerMinute:
            return (value / 60)
        
        if to_unit == TemperatureChangeRateUnits.NanodegreeCelsiusPerSecond:
            return ((value) * 1e-09)
        
        if to_unit == TemperatureChangeRateUnits.MicrodegreeCelsiusPerSecond:
            return ((value) * 1e-06)
        
        if to_unit == TemperatureChangeRateUnits.MillidegreeCelsiusPerSecond:
            return ((value) * 0.001)
        
        if to_unit == TemperatureChangeRateUnits.CentidegreeCelsiusPerSecond:
            return ((value) * 0.01)
        
        if to_unit == TemperatureChangeRateUnits.DecidegreeCelsiusPerSecond:
            return ((value) * 0.1)
        
        if to_unit == TemperatureChangeRateUnits.DecadegreeCelsiusPerSecond:
            return ((value) * 10.0)
        
        if to_unit == TemperatureChangeRateUnits.HectodegreeCelsiusPerSecond:
            return ((value) * 100.0)
        
        if to_unit == TemperatureChangeRateUnits.KilodegreeCelsiusPerSecond:
            return ((value) * 1000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_degrees_celsius_per_second(degrees_celsius_per_second: float):
        """
        Create a new instance of TemperatureChangeRate from a value in degrees_celsius_per_second.

        

        :param meters: The TemperatureChangeRate value in degrees_celsius_per_second.
        :type degrees_celsius_per_second: float
        :return: A new instance of TemperatureChangeRate.
        :rtype: TemperatureChangeRate
        """
        return TemperatureChangeRate(degrees_celsius_per_second, TemperatureChangeRateUnits.DegreeCelsiusPerSecond)

    
    @staticmethod
    def from_degrees_celsius_per_minute(degrees_celsius_per_minute: float):
        """
        Create a new instance of TemperatureChangeRate from a value in degrees_celsius_per_minute.

        

        :param meters: The TemperatureChangeRate value in degrees_celsius_per_minute.
        :type degrees_celsius_per_minute: float
        :return: A new instance of TemperatureChangeRate.
        :rtype: TemperatureChangeRate
        """
        return TemperatureChangeRate(degrees_celsius_per_minute, TemperatureChangeRateUnits.DegreeCelsiusPerMinute)

    
    @staticmethod
    def from_nanodegrees_celsius_per_second(nanodegrees_celsius_per_second: float):
        """
        Create a new instance of TemperatureChangeRate from a value in nanodegrees_celsius_per_second.

        

        :param meters: The TemperatureChangeRate value in nanodegrees_celsius_per_second.
        :type nanodegrees_celsius_per_second: float
        :return: A new instance of TemperatureChangeRate.
        :rtype: TemperatureChangeRate
        """
        return TemperatureChangeRate(nanodegrees_celsius_per_second, TemperatureChangeRateUnits.NanodegreeCelsiusPerSecond)

    
    @staticmethod
    def from_microdegrees_celsius_per_second(microdegrees_celsius_per_second: float):
        """
        Create a new instance of TemperatureChangeRate from a value in microdegrees_celsius_per_second.

        

        :param meters: The TemperatureChangeRate value in microdegrees_celsius_per_second.
        :type microdegrees_celsius_per_second: float
        :return: A new instance of TemperatureChangeRate.
        :rtype: TemperatureChangeRate
        """
        return TemperatureChangeRate(microdegrees_celsius_per_second, TemperatureChangeRateUnits.MicrodegreeCelsiusPerSecond)

    
    @staticmethod
    def from_millidegrees_celsius_per_second(millidegrees_celsius_per_second: float):
        """
        Create a new instance of TemperatureChangeRate from a value in millidegrees_celsius_per_second.

        

        :param meters: The TemperatureChangeRate value in millidegrees_celsius_per_second.
        :type millidegrees_celsius_per_second: float
        :return: A new instance of TemperatureChangeRate.
        :rtype: TemperatureChangeRate
        """
        return TemperatureChangeRate(millidegrees_celsius_per_second, TemperatureChangeRateUnits.MillidegreeCelsiusPerSecond)

    
    @staticmethod
    def from_centidegrees_celsius_per_second(centidegrees_celsius_per_second: float):
        """
        Create a new instance of TemperatureChangeRate from a value in centidegrees_celsius_per_second.

        

        :param meters: The TemperatureChangeRate value in centidegrees_celsius_per_second.
        :type centidegrees_celsius_per_second: float
        :return: A new instance of TemperatureChangeRate.
        :rtype: TemperatureChangeRate
        """
        return TemperatureChangeRate(centidegrees_celsius_per_second, TemperatureChangeRateUnits.CentidegreeCelsiusPerSecond)

    
    @staticmethod
    def from_decidegrees_celsius_per_second(decidegrees_celsius_per_second: float):
        """
        Create a new instance of TemperatureChangeRate from a value in decidegrees_celsius_per_second.

        

        :param meters: The TemperatureChangeRate value in decidegrees_celsius_per_second.
        :type decidegrees_celsius_per_second: float
        :return: A new instance of TemperatureChangeRate.
        :rtype: TemperatureChangeRate
        """
        return TemperatureChangeRate(decidegrees_celsius_per_second, TemperatureChangeRateUnits.DecidegreeCelsiusPerSecond)

    
    @staticmethod
    def from_decadegrees_celsius_per_second(decadegrees_celsius_per_second: float):
        """
        Create a new instance of TemperatureChangeRate from a value in decadegrees_celsius_per_second.

        

        :param meters: The TemperatureChangeRate value in decadegrees_celsius_per_second.
        :type decadegrees_celsius_per_second: float
        :return: A new instance of TemperatureChangeRate.
        :rtype: TemperatureChangeRate
        """
        return TemperatureChangeRate(decadegrees_celsius_per_second, TemperatureChangeRateUnits.DecadegreeCelsiusPerSecond)

    
    @staticmethod
    def from_hectodegrees_celsius_per_second(hectodegrees_celsius_per_second: float):
        """
        Create a new instance of TemperatureChangeRate from a value in hectodegrees_celsius_per_second.

        

        :param meters: The TemperatureChangeRate value in hectodegrees_celsius_per_second.
        :type hectodegrees_celsius_per_second: float
        :return: A new instance of TemperatureChangeRate.
        :rtype: TemperatureChangeRate
        """
        return TemperatureChangeRate(hectodegrees_celsius_per_second, TemperatureChangeRateUnits.HectodegreeCelsiusPerSecond)

    
    @staticmethod
    def from_kilodegrees_celsius_per_second(kilodegrees_celsius_per_second: float):
        """
        Create a new instance of TemperatureChangeRate from a value in kilodegrees_celsius_per_second.

        

        :param meters: The TemperatureChangeRate value in kilodegrees_celsius_per_second.
        :type kilodegrees_celsius_per_second: float
        :return: A new instance of TemperatureChangeRate.
        :rtype: TemperatureChangeRate
        """
        return TemperatureChangeRate(kilodegrees_celsius_per_second, TemperatureChangeRateUnits.KilodegreeCelsiusPerSecond)

    
    @property
    def degrees_celsius_per_second(self) -> float:
        """
        
        """
        if self.__degrees_celsius_per_second != None:
            return self.__degrees_celsius_per_second
        self.__degrees_celsius_per_second = self.__convert_from_base(TemperatureChangeRateUnits.DegreeCelsiusPerSecond)
        return self.__degrees_celsius_per_second

    
    @property
    def degrees_celsius_per_minute(self) -> float:
        """
        
        """
        if self.__degrees_celsius_per_minute != None:
            return self.__degrees_celsius_per_minute
        self.__degrees_celsius_per_minute = self.__convert_from_base(TemperatureChangeRateUnits.DegreeCelsiusPerMinute)
        return self.__degrees_celsius_per_minute

    
    @property
    def nanodegrees_celsius_per_second(self) -> float:
        """
        
        """
        if self.__nanodegrees_celsius_per_second != None:
            return self.__nanodegrees_celsius_per_second
        self.__nanodegrees_celsius_per_second = self.__convert_from_base(TemperatureChangeRateUnits.NanodegreeCelsiusPerSecond)
        return self.__nanodegrees_celsius_per_second

    
    @property
    def microdegrees_celsius_per_second(self) -> float:
        """
        
        """
        if self.__microdegrees_celsius_per_second != None:
            return self.__microdegrees_celsius_per_second
        self.__microdegrees_celsius_per_second = self.__convert_from_base(TemperatureChangeRateUnits.MicrodegreeCelsiusPerSecond)
        return self.__microdegrees_celsius_per_second

    
    @property
    def millidegrees_celsius_per_second(self) -> float:
        """
        
        """
        if self.__millidegrees_celsius_per_second != None:
            return self.__millidegrees_celsius_per_second
        self.__millidegrees_celsius_per_second = self.__convert_from_base(TemperatureChangeRateUnits.MillidegreeCelsiusPerSecond)
        return self.__millidegrees_celsius_per_second

    
    @property
    def centidegrees_celsius_per_second(self) -> float:
        """
        
        """
        if self.__centidegrees_celsius_per_second != None:
            return self.__centidegrees_celsius_per_second
        self.__centidegrees_celsius_per_second = self.__convert_from_base(TemperatureChangeRateUnits.CentidegreeCelsiusPerSecond)
        return self.__centidegrees_celsius_per_second

    
    @property
    def decidegrees_celsius_per_second(self) -> float:
        """
        
        """
        if self.__decidegrees_celsius_per_second != None:
            return self.__decidegrees_celsius_per_second
        self.__decidegrees_celsius_per_second = self.__convert_from_base(TemperatureChangeRateUnits.DecidegreeCelsiusPerSecond)
        return self.__decidegrees_celsius_per_second

    
    @property
    def decadegrees_celsius_per_second(self) -> float:
        """
        
        """
        if self.__decadegrees_celsius_per_second != None:
            return self.__decadegrees_celsius_per_second
        self.__decadegrees_celsius_per_second = self.__convert_from_base(TemperatureChangeRateUnits.DecadegreeCelsiusPerSecond)
        return self.__decadegrees_celsius_per_second

    
    @property
    def hectodegrees_celsius_per_second(self) -> float:
        """
        
        """
        if self.__hectodegrees_celsius_per_second != None:
            return self.__hectodegrees_celsius_per_second
        self.__hectodegrees_celsius_per_second = self.__convert_from_base(TemperatureChangeRateUnits.HectodegreeCelsiusPerSecond)
        return self.__hectodegrees_celsius_per_second

    
    @property
    def kilodegrees_celsius_per_second(self) -> float:
        """
        
        """
        if self.__kilodegrees_celsius_per_second != None:
            return self.__kilodegrees_celsius_per_second
        self.__kilodegrees_celsius_per_second = self.__convert_from_base(TemperatureChangeRateUnits.KilodegreeCelsiusPerSecond)
        return self.__kilodegrees_celsius_per_second

    
    def to_string(self, unit: TemperatureChangeRateUnits = TemperatureChangeRateUnits.DegreeCelsiusPerSecond, fractional_digits: int = None) -> str:
        """
        Format the TemperatureChangeRate to a string.
        
        Note: the default format for TemperatureChangeRate is DegreeCelsiusPerSecond.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the TemperatureChangeRate. Default is 'DegreeCelsiusPerSecond'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == TemperatureChangeRateUnits.DegreeCelsiusPerSecond:
            return f"""{super()._truncate_fraction_digits(self.degrees_celsius_per_second, fractional_digits)} °C/s"""
        
        if unit == TemperatureChangeRateUnits.DegreeCelsiusPerMinute:
            return f"""{super()._truncate_fraction_digits(self.degrees_celsius_per_minute, fractional_digits)} °C/min"""
        
        if unit == TemperatureChangeRateUnits.NanodegreeCelsiusPerSecond:
            return f"""{super()._truncate_fraction_digits(self.nanodegrees_celsius_per_second, fractional_digits)} n°C/s"""
        
        if unit == TemperatureChangeRateUnits.MicrodegreeCelsiusPerSecond:
            return f"""{super()._truncate_fraction_digits(self.microdegrees_celsius_per_second, fractional_digits)} μ°C/s"""
        
        if unit == TemperatureChangeRateUnits.MillidegreeCelsiusPerSecond:
            return f"""{super()._truncate_fraction_digits(self.millidegrees_celsius_per_second, fractional_digits)} m°C/s"""
        
        if unit == TemperatureChangeRateUnits.CentidegreeCelsiusPerSecond:
            return f"""{super()._truncate_fraction_digits(self.centidegrees_celsius_per_second, fractional_digits)} c°C/s"""
        
        if unit == TemperatureChangeRateUnits.DecidegreeCelsiusPerSecond:
            return f"""{super()._truncate_fraction_digits(self.decidegrees_celsius_per_second, fractional_digits)} d°C/s"""
        
        if unit == TemperatureChangeRateUnits.DecadegreeCelsiusPerSecond:
            return f"""{super()._truncate_fraction_digits(self.decadegrees_celsius_per_second, fractional_digits)} da°C/s"""
        
        if unit == TemperatureChangeRateUnits.HectodegreeCelsiusPerSecond:
            return f"""{super()._truncate_fraction_digits(self.hectodegrees_celsius_per_second, fractional_digits)} h°C/s"""
        
        if unit == TemperatureChangeRateUnits.KilodegreeCelsiusPerSecond:
            return f"""{super()._truncate_fraction_digits(self.kilodegrees_celsius_per_second, fractional_digits)} k°C/s"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: TemperatureChangeRateUnits = TemperatureChangeRateUnits.DegreeCelsiusPerSecond) -> str:
        """
        Get TemperatureChangeRate unit abbreviation.
        Note! the default abbreviation for TemperatureChangeRate is DegreeCelsiusPerSecond.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == TemperatureChangeRateUnits.DegreeCelsiusPerSecond:
            return """°C/s"""
        
        if unit_abbreviation == TemperatureChangeRateUnits.DegreeCelsiusPerMinute:
            return """°C/min"""
        
        if unit_abbreviation == TemperatureChangeRateUnits.NanodegreeCelsiusPerSecond:
            return """n°C/s"""
        
        if unit_abbreviation == TemperatureChangeRateUnits.MicrodegreeCelsiusPerSecond:
            return """μ°C/s"""
        
        if unit_abbreviation == TemperatureChangeRateUnits.MillidegreeCelsiusPerSecond:
            return """m°C/s"""
        
        if unit_abbreviation == TemperatureChangeRateUnits.CentidegreeCelsiusPerSecond:
            return """c°C/s"""
        
        if unit_abbreviation == TemperatureChangeRateUnits.DecidegreeCelsiusPerSecond:
            return """d°C/s"""
        
        if unit_abbreviation == TemperatureChangeRateUnits.DecadegreeCelsiusPerSecond:
            return """da°C/s"""
        
        if unit_abbreviation == TemperatureChangeRateUnits.HectodegreeCelsiusPerSecond:
            return """h°C/s"""
        
        if unit_abbreviation == TemperatureChangeRateUnits.KilodegreeCelsiusPerSecond:
            return """k°C/s"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class RelativeHumidityUnits(Enum):
        """
            RelativeHumidityUnits enumeration
        """
        
        Percent = 'Percent'
        """
            
        """
        

class RelativeHumidityDto:
    """
    A DTO representation of a RelativeHumidity

    Attributes:
        value (float): The value of the RelativeHumidity.
        unit (RelativeHumidityUnits): The specific unit that the RelativeHumidity value is representing.
    """

    def __init__(self, value: float, unit: RelativeHumidityUnits):
        """
        Create a new DTO representation of a RelativeHumidity

        Parameters:
            value (float): The value of the RelativeHumidity.
            unit (RelativeHumidityUnits): The specific unit that the RelativeHumidity value is representing.
        """
        self.value: float = value
        """
        The value of the RelativeHumidity
        """
        self.unit: RelativeHumidityUnits = unit
        """
        The specific unit that the RelativeHumidity value is representing
        """

    def to_json(self):
        """
        Get a RelativeHumidity DTO JSON object representing the current unit.

        :return: JSON object represents RelativeHumidity DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "Percent"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of RelativeHumidity DTO from a json representation.

        :param data: The RelativeHumidity DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "Percent"}
        :return: A new instance of RelativeHumidityDto.
        :rtype: RelativeHumidityDto
        """
        return RelativeHumidityDto(value=data["value"], unit=RelativeHumidityUnits(data["unit"]))


class RelativeHumidity(AbstractMeasure):
    """
    Relative humidity is a ratio of the actual water vapor present in the air to the maximum water vapor in the air at the given temperature.

    Args:
        value (float): The value.
        from_unit (RelativeHumidityUnits): The RelativeHumidity unit to create from, The default unit is Percent
    """
    def __init__(self, value: float, from_unit: RelativeHumidityUnits = RelativeHumidityUnits.Percent):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__percent = None
        

    def convert(self, unit: RelativeHumidityUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: RelativeHumidityUnits = RelativeHumidityUnits.Percent) -> RelativeHumidityDto:
        """
        Get a new instance of RelativeHumidity DTO representing the current unit.

        :param hold_in_unit: The specific RelativeHumidity unit to store the RelativeHumidity value in the DTO representation.
        :type hold_in_unit: RelativeHumidityUnits
        :return: A new instance of RelativeHumidityDto.
        :rtype: RelativeHumidityDto
        """
        return RelativeHumidityDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: RelativeHumidityUnits = RelativeHumidityUnits.Percent):
        """
        Get a RelativeHumidity DTO JSON object representing the current unit.

        :param hold_in_unit: The specific RelativeHumidity unit to store the RelativeHumidity value in the DTO representation.
        :type hold_in_unit: RelativeHumidityUnits
        :return: JSON object represents RelativeHumidity DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "Percent"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(relative_humidity_dto: RelativeHumidityDto):
        """
        Obtain a new instance of RelativeHumidity from a DTO unit object.

        :param relative_humidity_dto: The RelativeHumidity DTO representation.
        :type relative_humidity_dto: RelativeHumidityDto
        :return: A new instance of RelativeHumidity.
        :rtype: RelativeHumidity
        """
        return RelativeHumidity(relative_humidity_dto.value, relative_humidity_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of RelativeHumidity from a DTO unit json representation.

        :param data: The RelativeHumidity DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "Percent"}
        :return: A new instance of RelativeHumidity.
        :rtype: RelativeHumidity
        """
        return RelativeHumidity.from_dto(RelativeHumidityDto.from_json(data))

    def __convert_from_base(self, from_unit: RelativeHumidityUnits) -> float:
        value = self._value
        
        if from_unit == RelativeHumidityUnits.Percent:
            return (value)
        
        return None


    def __convert_to_base(self, value: float, to_unit: RelativeHumidityUnits) -> float:
        
        if to_unit == RelativeHumidityUnits.Percent:
            return (value)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_percent(percent: float):
        """
        Create a new instance of RelativeHumidity from a value in percent.

        

        :param meters: The RelativeHumidity value in percent.
        :type percent: float
        :return: A new instance of RelativeHumidity.
        :rtype: RelativeHumidity
        """
        return RelativeHumidity(percent, RelativeHumidityUnits.Percent)

    
    @property
    def percent(self) -> float:
        """
        
        """
        if self.__percent != None:
            return self.__percent
        self.__percent = self.__convert_from_base(RelativeHumidityUnits.Percent)
        return self.__percent

    
    def to_string(self, unit: RelativeHumidityUnits = RelativeHumidityUnits.Percent, fractional_digits: int = None) -> str:
        """
        Format the RelativeHumidity to a string.
        
        Note: the default format for RelativeHumidity is Percent.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the RelativeHumidity. Default is 'Percent'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == RelativeHumidityUnits.Percent:
            return f"""{super()._truncate_fraction_digits(self.percent, fractional_digits)} %RH"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: RelativeHumidityUnits = RelativeHumidityUnits.Percent) -> str:
        """
        Get RelativeHumidity unit abbreviation.
        Note! the default abbreviation for RelativeHumidity is Percent.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == RelativeHumidityUnits.Percent:
            return """%RH"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class RadiationExposureUnits(Enum):
        """
            RadiationExposureUnits enumeration
        """
        
        CoulombPerKilogram = 'CoulombPerKilogram'
        """
            
        """
        
        Roentgen = 'Roentgen'
        """
            
        """
        
        PicocoulombPerKilogram = 'PicocoulombPerKilogram'
        """
            
        """
        
        NanocoulombPerKilogram = 'NanocoulombPerKilogram'
        """
            
        """
        
        MicrocoulombPerKilogram = 'MicrocoulombPerKilogram'
        """
            
        """
        
        MillicoulombPerKilogram = 'MillicoulombPerKilogram'
        """
            
        """
        
        Microroentgen = 'Microroentgen'
        """
            
        """
        
        Milliroentgen = 'Milliroentgen'
        """
            
        """
        

class RadiationExposureDto:
    """
    A DTO representation of a RadiationExposure

    Attributes:
        value (float): The value of the RadiationExposure.
        unit (RadiationExposureUnits): The specific unit that the RadiationExposure value is representing.
    """

    def __init__(self, value: float, unit: RadiationExposureUnits):
        """
        Create a new DTO representation of a RadiationExposure

        Parameters:
            value (float): The value of the RadiationExposure.
            unit (RadiationExposureUnits): The specific unit that the RadiationExposure value is representing.
        """
        self.value: float = value
        """
        The value of the RadiationExposure
        """
        self.unit: RadiationExposureUnits = unit
        """
        The specific unit that the RadiationExposure value is representing
        """

    def to_json(self):
        """
        Get a RadiationExposure DTO JSON object representing the current unit.

        :return: JSON object represents RadiationExposure DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "CoulombPerKilogram"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of RadiationExposure DTO from a json representation.

        :param data: The RadiationExposure DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "CoulombPerKilogram"}
        :return: A new instance of RadiationExposureDto.
        :rtype: RadiationExposureDto
        """
        return RadiationExposureDto(value=data["value"], unit=RadiationExposureUnits(data["unit"]))


class RadiationExposure(AbstractMeasure):
    """
    Radiation exposure is a measure of the ionization of air due to ionizing radiation from photons.

    Args:
        value (float): The value.
        from_unit (RadiationExposureUnits): The RadiationExposure unit to create from, The default unit is CoulombPerKilogram
    """
    def __init__(self, value: float, from_unit: RadiationExposureUnits = RadiationExposureUnits.CoulombPerKilogram):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__coulombs_per_kilogram = None
        
        self.__roentgens = None
        
        self.__picocoulombs_per_kilogram = None
        
        self.__nanocoulombs_per_kilogram = None
        
        self.__microcoulombs_per_kilogram = None
        
        self.__millicoulombs_per_kilogram = None
        
        self.__microroentgens = None
        
        self.__milliroentgens = None
        

    def convert(self, unit: RadiationExposureUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: RadiationExposureUnits = RadiationExposureUnits.CoulombPerKilogram) -> RadiationExposureDto:
        """
        Get a new instance of RadiationExposure DTO representing the current unit.

        :param hold_in_unit: The specific RadiationExposure unit to store the RadiationExposure value in the DTO representation.
        :type hold_in_unit: RadiationExposureUnits
        :return: A new instance of RadiationExposureDto.
        :rtype: RadiationExposureDto
        """
        return RadiationExposureDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: RadiationExposureUnits = RadiationExposureUnits.CoulombPerKilogram):
        """
        Get a RadiationExposure DTO JSON object representing the current unit.

        :param hold_in_unit: The specific RadiationExposure unit to store the RadiationExposure value in the DTO representation.
        :type hold_in_unit: RadiationExposureUnits
        :return: JSON object represents RadiationExposure DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "CoulombPerKilogram"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(radiation_exposure_dto: RadiationExposureDto):
        """
        Obtain a new instance of RadiationExposure from a DTO unit object.

        :param radiation_exposure_dto: The RadiationExposure DTO representation.
        :type radiation_exposure_dto: RadiationExposureDto
        :return: A new instance of RadiationExposure.
        :rtype: RadiationExposure
        """
        return RadiationExposure(radiation_exposure_dto.value, radiation_exposure_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of RadiationExposure from a DTO unit json representation.

        :param data: The RadiationExposure DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "CoulombPerKilogram"}
        :return: A new instance of RadiationExposure.
        :rtype: RadiationExposure
        """
        return RadiationExposure.from_dto(RadiationExposureDto.from_json(data))

    def __convert_from_base(self, from_unit: RadiationExposureUnits) -> float:
        value = self._value
        
        if from_unit == RadiationExposureUnits.CoulombPerKilogram:
            return (value)
        
        if from_unit == RadiationExposureUnits.Roentgen:
            return (value / 2.58e-4)
        
        if from_unit == RadiationExposureUnits.PicocoulombPerKilogram:
            return ((value) / 1e-12)
        
        if from_unit == RadiationExposureUnits.NanocoulombPerKilogram:
            return ((value) / 1e-09)
        
        if from_unit == RadiationExposureUnits.MicrocoulombPerKilogram:
            return ((value) / 1e-06)
        
        if from_unit == RadiationExposureUnits.MillicoulombPerKilogram:
            return ((value) / 0.001)
        
        if from_unit == RadiationExposureUnits.Microroentgen:
            return ((value / 2.58e-4) / 1e-06)
        
        if from_unit == RadiationExposureUnits.Milliroentgen:
            return ((value / 2.58e-4) / 0.001)
        
        return None


    def __convert_to_base(self, value: float, to_unit: RadiationExposureUnits) -> float:
        
        if to_unit == RadiationExposureUnits.CoulombPerKilogram:
            return (value)
        
        if to_unit == RadiationExposureUnits.Roentgen:
            return (value * 2.58e-4)
        
        if to_unit == RadiationExposureUnits.PicocoulombPerKilogram:
            return ((value) * 1e-12)
        
        if to_unit == RadiationExposureUnits.NanocoulombPerKilogram:
            return ((value) * 1e-09)
        
        if to_unit == RadiationExposureUnits.MicrocoulombPerKilogram:
            return ((value) * 1e-06)
        
        if to_unit == RadiationExposureUnits.MillicoulombPerKilogram:
            return ((value) * 0.001)
        
        if to_unit == RadiationExposureUnits.Microroentgen:
            return ((value * 2.58e-4) * 1e-06)
        
        if to_unit == RadiationExposureUnits.Milliroentgen:
            return ((value * 2.58e-4) * 0.001)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_coulombs_per_kilogram(coulombs_per_kilogram: float):
        """
        Create a new instance of RadiationExposure from a value in coulombs_per_kilogram.

        

        :param meters: The RadiationExposure value in coulombs_per_kilogram.
        :type coulombs_per_kilogram: float
        :return: A new instance of RadiationExposure.
        :rtype: RadiationExposure
        """
        return RadiationExposure(coulombs_per_kilogram, RadiationExposureUnits.CoulombPerKilogram)

    
    @staticmethod
    def from_roentgens(roentgens: float):
        """
        Create a new instance of RadiationExposure from a value in roentgens.

        

        :param meters: The RadiationExposure value in roentgens.
        :type roentgens: float
        :return: A new instance of RadiationExposure.
        :rtype: RadiationExposure
        """
        return RadiationExposure(roentgens, RadiationExposureUnits.Roentgen)

    
    @staticmethod
    def from_picocoulombs_per_kilogram(picocoulombs_per_kilogram: float):
        """
        Create a new instance of RadiationExposure from a value in picocoulombs_per_kilogram.

        

        :param meters: The RadiationExposure value in picocoulombs_per_kilogram.
        :type picocoulombs_per_kilogram: float
        :return: A new instance of RadiationExposure.
        :rtype: RadiationExposure
        """
        return RadiationExposure(picocoulombs_per_kilogram, RadiationExposureUnits.PicocoulombPerKilogram)

    
    @staticmethod
    def from_nanocoulombs_per_kilogram(nanocoulombs_per_kilogram: float):
        """
        Create a new instance of RadiationExposure from a value in nanocoulombs_per_kilogram.

        

        :param meters: The RadiationExposure value in nanocoulombs_per_kilogram.
        :type nanocoulombs_per_kilogram: float
        :return: A new instance of RadiationExposure.
        :rtype: RadiationExposure
        """
        return RadiationExposure(nanocoulombs_per_kilogram, RadiationExposureUnits.NanocoulombPerKilogram)

    
    @staticmethod
    def from_microcoulombs_per_kilogram(microcoulombs_per_kilogram: float):
        """
        Create a new instance of RadiationExposure from a value in microcoulombs_per_kilogram.

        

        :param meters: The RadiationExposure value in microcoulombs_per_kilogram.
        :type microcoulombs_per_kilogram: float
        :return: A new instance of RadiationExposure.
        :rtype: RadiationExposure
        """
        return RadiationExposure(microcoulombs_per_kilogram, RadiationExposureUnits.MicrocoulombPerKilogram)

    
    @staticmethod
    def from_millicoulombs_per_kilogram(millicoulombs_per_kilogram: float):
        """
        Create a new instance of RadiationExposure from a value in millicoulombs_per_kilogram.

        

        :param meters: The RadiationExposure value in millicoulombs_per_kilogram.
        :type millicoulombs_per_kilogram: float
        :return: A new instance of RadiationExposure.
        :rtype: RadiationExposure
        """
        return RadiationExposure(millicoulombs_per_kilogram, RadiationExposureUnits.MillicoulombPerKilogram)

    
    @staticmethod
    def from_microroentgens(microroentgens: float):
        """
        Create a new instance of RadiationExposure from a value in microroentgens.

        

        :param meters: The RadiationExposure value in microroentgens.
        :type microroentgens: float
        :return: A new instance of RadiationExposure.
        :rtype: RadiationExposure
        """
        return RadiationExposure(microroentgens, RadiationExposureUnits.Microroentgen)

    
    @staticmethod
    def from_milliroentgens(milliroentgens: float):
        """
        Create a new instance of RadiationExposure from a value in milliroentgens.

        

        :param meters: The RadiationExposure value in milliroentgens.
        :type milliroentgens: float
        :return: A new instance of RadiationExposure.
        :rtype: RadiationExposure
        """
        return RadiationExposure(milliroentgens, RadiationExposureUnits.Milliroentgen)

    
    @property
    def coulombs_per_kilogram(self) -> float:
        """
        
        """
        if self.__coulombs_per_kilogram != None:
            return self.__coulombs_per_kilogram
        self.__coulombs_per_kilogram = self.__convert_from_base(RadiationExposureUnits.CoulombPerKilogram)
        return self.__coulombs_per_kilogram

    
    @property
    def roentgens(self) -> float:
        """
        
        """
        if self.__roentgens != None:
            return self.__roentgens
        self.__roentgens = self.__convert_from_base(RadiationExposureUnits.Roentgen)
        return self.__roentgens

    
    @property
    def picocoulombs_per_kilogram(self) -> float:
        """
        
        """
        if self.__picocoulombs_per_kilogram != None:
            return self.__picocoulombs_per_kilogram
        self.__picocoulombs_per_kilogram = self.__convert_from_base(RadiationExposureUnits.PicocoulombPerKilogram)
        return self.__picocoulombs_per_kilogram

    
    @property
    def nanocoulombs_per_kilogram(self) -> float:
        """
        
        """
        if self.__nanocoulombs_per_kilogram != None:
            return self.__nanocoulombs_per_kilogram
        self.__nanocoulombs_per_kilogram = self.__convert_from_base(RadiationExposureUnits.NanocoulombPerKilogram)
        return self.__nanocoulombs_per_kilogram

    
    @property
    def microcoulombs_per_kilogram(self) -> float:
        """
        
        """
        if self.__microcoulombs_per_kilogram != None:
            return self.__microcoulombs_per_kilogram
        self.__microcoulombs_per_kilogram = self.__convert_from_base(RadiationExposureUnits.MicrocoulombPerKilogram)
        return self.__microcoulombs_per_kilogram

    
    @property
    def millicoulombs_per_kilogram(self) -> float:
        """
        
        """
        if self.__millicoulombs_per_kilogram != None:
            return self.__millicoulombs_per_kilogram
        self.__millicoulombs_per_kilogram = self.__convert_from_base(RadiationExposureUnits.MillicoulombPerKilogram)
        return self.__millicoulombs_per_kilogram

    
    @property
    def microroentgens(self) -> float:
        """
        
        """
        if self.__microroentgens != None:
            return self.__microroentgens
        self.__microroentgens = self.__convert_from_base(RadiationExposureUnits.Microroentgen)
        return self.__microroentgens

    
    @property
    def milliroentgens(self) -> float:
        """
        
        """
        if self.__milliroentgens != None:
            return self.__milliroentgens
        self.__milliroentgens = self.__convert_from_base(RadiationExposureUnits.Milliroentgen)
        return self.__milliroentgens

    
    def to_string(self, unit: RadiationExposureUnits = RadiationExposureUnits.CoulombPerKilogram, fractional_digits: int = None) -> str:
        """
        Format the RadiationExposure to a string.
        
        Note: the default format for RadiationExposure is CoulombPerKilogram.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the RadiationExposure. Default is 'CoulombPerKilogram'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == RadiationExposureUnits.CoulombPerKilogram:
            return f"""{super()._truncate_fraction_digits(self.coulombs_per_kilogram, fractional_digits)} C/kg"""
        
        if unit == RadiationExposureUnits.Roentgen:
            return f"""{super()._truncate_fraction_digits(self.roentgens, fractional_digits)} R"""
        
        if unit == RadiationExposureUnits.PicocoulombPerKilogram:
            return f"""{super()._truncate_fraction_digits(self.picocoulombs_per_kilogram, fractional_digits)} pC/kg"""
        
        if unit == RadiationExposureUnits.NanocoulombPerKilogram:
            return f"""{super()._truncate_fraction_digits(self.nanocoulombs_per_kilogram, fractional_digits)} nC/kg"""
        
        if unit == RadiationExposureUnits.MicrocoulombPerKilogram:
            return f"""{super()._truncate_fraction_digits(self.microcoulombs_per_kilogram, fractional_digits)} μC/kg"""
        
        if unit == RadiationExposureUnits.MillicoulombPerKilogram:
            return f"""{super()._truncate_fraction_digits(self.millicoulombs_per_kilogram, fractional_digits)} mC/kg"""
        
        if unit == RadiationExposureUnits.Microroentgen:
            return f"""{super()._truncate_fraction_digits(self.microroentgens, fractional_digits)} μR"""
        
        if unit == RadiationExposureUnits.Milliroentgen:
            return f"""{super()._truncate_fraction_digits(self.milliroentgens, fractional_digits)} mR"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: RadiationExposureUnits = RadiationExposureUnits.CoulombPerKilogram) -> str:
        """
        Get RadiationExposure unit abbreviation.
        Note! the default abbreviation for RadiationExposure is CoulombPerKilogram.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == RadiationExposureUnits.CoulombPerKilogram:
            return """C/kg"""
        
        if unit_abbreviation == RadiationExposureUnits.Roentgen:
            return """R"""
        
        if unit_abbreviation == RadiationExposureUnits.PicocoulombPerKilogram:
            return """pC/kg"""
        
        if unit_abbreviation == RadiationExposureUnits.NanocoulombPerKilogram:
            return """nC/kg"""
        
        if unit_abbreviation == RadiationExposureUnits.MicrocoulombPerKilogram:
            return """μC/kg"""
        
        if unit_abbreviation == RadiationExposureUnits.MillicoulombPerKilogram:
            return """mC/kg"""
        
        if unit_abbreviation == RadiationExposureUnits.Microroentgen:
            return """μR"""
        
        if unit_abbreviation == RadiationExposureUnits.Milliroentgen:
            return """mR"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class MolarEnergyUnits(Enum):
        """
            MolarEnergyUnits enumeration
        """
        
        JoulePerMole = 'JoulePerMole'
        """
            
        """
        
        KilojoulePerMole = 'KilojoulePerMole'
        """
            
        """
        
        MegajoulePerMole = 'MegajoulePerMole'
        """
            
        """
        

class MolarEnergyDto:
    """
    A DTO representation of a MolarEnergy

    Attributes:
        value (float): The value of the MolarEnergy.
        unit (MolarEnergyUnits): The specific unit that the MolarEnergy value is representing.
    """

    def __init__(self, value: float, unit: MolarEnergyUnits):
        """
        Create a new DTO representation of a MolarEnergy

        Parameters:
            value (float): The value of the MolarEnergy.
            unit (MolarEnergyUnits): The specific unit that the MolarEnergy value is representing.
        """
        self.value: float = value
        """
        The value of the MolarEnergy
        """
        self.unit: MolarEnergyUnits = unit
        """
        The specific unit that the MolarEnergy value is representing
        """

    def to_json(self):
        """
        Get a MolarEnergy DTO JSON object representing the current unit.

        :return: JSON object represents MolarEnergy DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "JoulePerMole"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of MolarEnergy DTO from a json representation.

        :param data: The MolarEnergy DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "JoulePerMole"}
        :return: A new instance of MolarEnergyDto.
        :rtype: MolarEnergyDto
        """
        return MolarEnergyDto(value=data["value"], unit=MolarEnergyUnits(data["unit"]))


class MolarEnergy(AbstractMeasure):
    """
    Molar energy is the amount of energy stored in 1 mole of a substance.

    Args:
        value (float): The value.
        from_unit (MolarEnergyUnits): The MolarEnergy unit to create from, The default unit is JoulePerMole
    """
    def __init__(self, value: float, from_unit: MolarEnergyUnits = MolarEnergyUnits.JoulePerMole):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__joules_per_mole = None
        
        self.__kilojoules_per_mole = None
        
        self.__megajoules_per_mole = None
        

    def convert(self, unit: MolarEnergyUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: MolarEnergyUnits = MolarEnergyUnits.JoulePerMole) -> MolarEnergyDto:
        """
        Get a new instance of MolarEnergy DTO representing the current unit.

        :param hold_in_unit: The specific MolarEnergy unit to store the MolarEnergy value in the DTO representation.
        :type hold_in_unit: MolarEnergyUnits
        :return: A new instance of MolarEnergyDto.
        :rtype: MolarEnergyDto
        """
        return MolarEnergyDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: MolarEnergyUnits = MolarEnergyUnits.JoulePerMole):
        """
        Get a MolarEnergy DTO JSON object representing the current unit.

        :param hold_in_unit: The specific MolarEnergy unit to store the MolarEnergy value in the DTO representation.
        :type hold_in_unit: MolarEnergyUnits
        :return: JSON object represents MolarEnergy DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "JoulePerMole"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(molar_energy_dto: MolarEnergyDto):
        """
        Obtain a new instance of MolarEnergy from a DTO unit object.

        :param molar_energy_dto: The MolarEnergy DTO representation.
        :type molar_energy_dto: MolarEnergyDto
        :return: A new instance of MolarEnergy.
        :rtype: MolarEnergy
        """
        return MolarEnergy(molar_energy_dto.value, molar_energy_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of MolarEnergy from a DTO unit json representation.

        :param data: The MolarEnergy DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "JoulePerMole"}
        :return: A new instance of MolarEnergy.
        :rtype: MolarEnergy
        """
        return MolarEnergy.from_dto(MolarEnergyDto.from_json(data))

    def __convert_from_base(self, from_unit: MolarEnergyUnits) -> float:
        value = self._value
        
        if from_unit == MolarEnergyUnits.JoulePerMole:
            return (value)
        
        if from_unit == MolarEnergyUnits.KilojoulePerMole:
            return ((value) / 1000.0)
        
        if from_unit == MolarEnergyUnits.MegajoulePerMole:
            return ((value) / 1000000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: MolarEnergyUnits) -> float:
        
        if to_unit == MolarEnergyUnits.JoulePerMole:
            return (value)
        
        if to_unit == MolarEnergyUnits.KilojoulePerMole:
            return ((value) * 1000.0)
        
        if to_unit == MolarEnergyUnits.MegajoulePerMole:
            return ((value) * 1000000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_joules_per_mole(joules_per_mole: float):
        """
        Create a new instance of MolarEnergy from a value in joules_per_mole.

        

        :param meters: The MolarEnergy value in joules_per_mole.
        :type joules_per_mole: float
        :return: A new instance of MolarEnergy.
        :rtype: MolarEnergy
        """
        return MolarEnergy(joules_per_mole, MolarEnergyUnits.JoulePerMole)

    
    @staticmethod
    def from_kilojoules_per_mole(kilojoules_per_mole: float):
        """
        Create a new instance of MolarEnergy from a value in kilojoules_per_mole.

        

        :param meters: The MolarEnergy value in kilojoules_per_mole.
        :type kilojoules_per_mole: float
        :return: A new instance of MolarEnergy.
        :rtype: MolarEnergy
        """
        return MolarEnergy(kilojoules_per_mole, MolarEnergyUnits.KilojoulePerMole)

    
    @staticmethod
    def from_megajoules_per_mole(megajoules_per_mole: float):
        """
        Create a new instance of MolarEnergy from a value in megajoules_per_mole.

        

        :param meters: The MolarEnergy value in megajoules_per_mole.
        :type megajoules_per_mole: float
        :return: A new instance of MolarEnergy.
        :rtype: MolarEnergy
        """
        return MolarEnergy(megajoules_per_mole, MolarEnergyUnits.MegajoulePerMole)

    
    @property
    def joules_per_mole(self) -> float:
        """
        
        """
        if self.__joules_per_mole != None:
            return self.__joules_per_mole
        self.__joules_per_mole = self.__convert_from_base(MolarEnergyUnits.JoulePerMole)
        return self.__joules_per_mole

    
    @property
    def kilojoules_per_mole(self) -> float:
        """
        
        """
        if self.__kilojoules_per_mole != None:
            return self.__kilojoules_per_mole
        self.__kilojoules_per_mole = self.__convert_from_base(MolarEnergyUnits.KilojoulePerMole)
        return self.__kilojoules_per_mole

    
    @property
    def megajoules_per_mole(self) -> float:
        """
        
        """
        if self.__megajoules_per_mole != None:
            return self.__megajoules_per_mole
        self.__megajoules_per_mole = self.__convert_from_base(MolarEnergyUnits.MegajoulePerMole)
        return self.__megajoules_per_mole

    
    def to_string(self, unit: MolarEnergyUnits = MolarEnergyUnits.JoulePerMole, fractional_digits: int = None) -> str:
        """
        Format the MolarEnergy to a string.
        
        Note: the default format for MolarEnergy is JoulePerMole.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the MolarEnergy. Default is 'JoulePerMole'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == MolarEnergyUnits.JoulePerMole:
            return f"""{super()._truncate_fraction_digits(self.joules_per_mole, fractional_digits)} J/mol"""
        
        if unit == MolarEnergyUnits.KilojoulePerMole:
            return f"""{super()._truncate_fraction_digits(self.kilojoules_per_mole, fractional_digits)} kJ/mol"""
        
        if unit == MolarEnergyUnits.MegajoulePerMole:
            return f"""{super()._truncate_fraction_digits(self.megajoules_per_mole, fractional_digits)} MJ/mol"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: MolarEnergyUnits = MolarEnergyUnits.JoulePerMole) -> str:
        """
        Get MolarEnergy unit abbreviation.
        Note! the default abbreviation for MolarEnergy is JoulePerMole.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == MolarEnergyUnits.JoulePerMole:
            return """J/mol"""
        
        if unit_abbreviation == MolarEnergyUnits.KilojoulePerMole:
            return """kJ/mol"""
        
        if unit_abbreviation == MolarEnergyUnits.MegajoulePerMole:
            return """MJ/mol"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class MagneticFluxUnits(Enum):
        """
            MagneticFluxUnits enumeration
        """
        
        Weber = 'Weber'
        """
            
        """
        

class MagneticFluxDto:
    """
    A DTO representation of a MagneticFlux

    Attributes:
        value (float): The value of the MagneticFlux.
        unit (MagneticFluxUnits): The specific unit that the MagneticFlux value is representing.
    """

    def __init__(self, value: float, unit: MagneticFluxUnits):
        """
        Create a new DTO representation of a MagneticFlux

        Parameters:
            value (float): The value of the MagneticFlux.
            unit (MagneticFluxUnits): The specific unit that the MagneticFlux value is representing.
        """
        self.value: float = value
        """
        The value of the MagneticFlux
        """
        self.unit: MagneticFluxUnits = unit
        """
        The specific unit that the MagneticFlux value is representing
        """

    def to_json(self):
        """
        Get a MagneticFlux DTO JSON object representing the current unit.

        :return: JSON object represents MagneticFlux DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "Weber"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of MagneticFlux DTO from a json representation.

        :param data: The MagneticFlux DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "Weber"}
        :return: A new instance of MagneticFluxDto.
        :rtype: MagneticFluxDto
        """
        return MagneticFluxDto(value=data["value"], unit=MagneticFluxUnits(data["unit"]))


class MagneticFlux(AbstractMeasure):
    """
    In physics, specifically electromagnetism, the magnetic flux through a surface is the surface integral of the normal component of the magnetic field B passing through that surface.

    Args:
        value (float): The value.
        from_unit (MagneticFluxUnits): The MagneticFlux unit to create from, The default unit is Weber
    """
    def __init__(self, value: float, from_unit: MagneticFluxUnits = MagneticFluxUnits.Weber):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__webers = None
        

    def convert(self, unit: MagneticFluxUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: MagneticFluxUnits = MagneticFluxUnits.Weber) -> MagneticFluxDto:
        """
        Get a new instance of MagneticFlux DTO representing the current unit.

        :param hold_in_unit: The specific MagneticFlux unit to store the MagneticFlux value in the DTO representation.
        :type hold_in_unit: MagneticFluxUnits
        :return: A new instance of MagneticFluxDto.
        :rtype: MagneticFluxDto
        """
        return MagneticFluxDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: MagneticFluxUnits = MagneticFluxUnits.Weber):
        """
        Get a MagneticFlux DTO JSON object representing the current unit.

        :param hold_in_unit: The specific MagneticFlux unit to store the MagneticFlux value in the DTO representation.
        :type hold_in_unit: MagneticFluxUnits
        :return: JSON object represents MagneticFlux DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "Weber"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(magnetic_flux_dto: MagneticFluxDto):
        """
        Obtain a new instance of MagneticFlux from a DTO unit object.

        :param magnetic_flux_dto: The MagneticFlux DTO representation.
        :type magnetic_flux_dto: MagneticFluxDto
        :return: A new instance of MagneticFlux.
        :rtype: MagneticFlux
        """
        return MagneticFlux(magnetic_flux_dto.value, magnetic_flux_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of MagneticFlux from a DTO unit json representation.

        :param data: The MagneticFlux DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "Weber"}
        :return: A new instance of MagneticFlux.
        :rtype: MagneticFlux
        """
        return MagneticFlux.from_dto(MagneticFluxDto.from_json(data))

    def __convert_from_base(self, from_unit: MagneticFluxUnits) -> float:
        value = self._value
        
        if from_unit == MagneticFluxUnits.Weber:
            return (value)
        
        return None


    def __convert_to_base(self, value: float, to_unit: MagneticFluxUnits) -> float:
        
        if to_unit == MagneticFluxUnits.Weber:
            return (value)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_webers(webers: float):
        """
        Create a new instance of MagneticFlux from a value in webers.

        

        :param meters: The MagneticFlux value in webers.
        :type webers: float
        :return: A new instance of MagneticFlux.
        :rtype: MagneticFlux
        """
        return MagneticFlux(webers, MagneticFluxUnits.Weber)

    
    @property
    def webers(self) -> float:
        """
        
        """
        if self.__webers != None:
            return self.__webers
        self.__webers = self.__convert_from_base(MagneticFluxUnits.Weber)
        return self.__webers

    
    def to_string(self, unit: MagneticFluxUnits = MagneticFluxUnits.Weber, fractional_digits: int = None) -> str:
        """
        Format the MagneticFlux to a string.
        
        Note: the default format for MagneticFlux is Weber.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the MagneticFlux. Default is 'Weber'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == MagneticFluxUnits.Weber:
            return f"""{super()._truncate_fraction_digits(self.webers, fractional_digits)} Wb"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: MagneticFluxUnits = MagneticFluxUnits.Weber) -> str:
        """
        Get MagneticFlux unit abbreviation.
        Note! the default abbreviation for MagneticFlux is Weber.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == MagneticFluxUnits.Weber:
            return """Wb"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class PorousMediumPermeabilityUnits(Enum):
        """
            PorousMediumPermeabilityUnits enumeration
        """
        
        Darcy = 'Darcy'
        """
            
        """
        
        SquareMeter = 'SquareMeter'
        """
            
        """
        
        SquareCentimeter = 'SquareCentimeter'
        """
            
        """
        
        Microdarcy = 'Microdarcy'
        """
            
        """
        
        Millidarcy = 'Millidarcy'
        """
            
        """
        

class PorousMediumPermeabilityDto:
    """
    A DTO representation of a PorousMediumPermeability

    Attributes:
        value (float): The value of the PorousMediumPermeability.
        unit (PorousMediumPermeabilityUnits): The specific unit that the PorousMediumPermeability value is representing.
    """

    def __init__(self, value: float, unit: PorousMediumPermeabilityUnits):
        """
        Create a new DTO representation of a PorousMediumPermeability

        Parameters:
            value (float): The value of the PorousMediumPermeability.
            unit (PorousMediumPermeabilityUnits): The specific unit that the PorousMediumPermeability value is representing.
        """
        self.value: float = value
        """
        The value of the PorousMediumPermeability
        """
        self.unit: PorousMediumPermeabilityUnits = unit
        """
        The specific unit that the PorousMediumPermeability value is representing
        """

    def to_json(self):
        """
        Get a PorousMediumPermeability DTO JSON object representing the current unit.

        :return: JSON object represents PorousMediumPermeability DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "SquareMeter"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of PorousMediumPermeability DTO from a json representation.

        :param data: The PorousMediumPermeability DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "SquareMeter"}
        :return: A new instance of PorousMediumPermeabilityDto.
        :rtype: PorousMediumPermeabilityDto
        """
        return PorousMediumPermeabilityDto(value=data["value"], unit=PorousMediumPermeabilityUnits(data["unit"]))


class PorousMediumPermeability(AbstractMeasure):
    """
    None

    Args:
        value (float): The value.
        from_unit (PorousMediumPermeabilityUnits): The PorousMediumPermeability unit to create from, The default unit is SquareMeter
    """
    def __init__(self, value: float, from_unit: PorousMediumPermeabilityUnits = PorousMediumPermeabilityUnits.SquareMeter):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__darcys = None
        
        self.__square_meters = None
        
        self.__square_centimeters = None
        
        self.__microdarcys = None
        
        self.__millidarcys = None
        

    def convert(self, unit: PorousMediumPermeabilityUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: PorousMediumPermeabilityUnits = PorousMediumPermeabilityUnits.SquareMeter) -> PorousMediumPermeabilityDto:
        """
        Get a new instance of PorousMediumPermeability DTO representing the current unit.

        :param hold_in_unit: The specific PorousMediumPermeability unit to store the PorousMediumPermeability value in the DTO representation.
        :type hold_in_unit: PorousMediumPermeabilityUnits
        :return: A new instance of PorousMediumPermeabilityDto.
        :rtype: PorousMediumPermeabilityDto
        """
        return PorousMediumPermeabilityDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: PorousMediumPermeabilityUnits = PorousMediumPermeabilityUnits.SquareMeter):
        """
        Get a PorousMediumPermeability DTO JSON object representing the current unit.

        :param hold_in_unit: The specific PorousMediumPermeability unit to store the PorousMediumPermeability value in the DTO representation.
        :type hold_in_unit: PorousMediumPermeabilityUnits
        :return: JSON object represents PorousMediumPermeability DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "SquareMeter"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(porous_medium_permeability_dto: PorousMediumPermeabilityDto):
        """
        Obtain a new instance of PorousMediumPermeability from a DTO unit object.

        :param porous_medium_permeability_dto: The PorousMediumPermeability DTO representation.
        :type porous_medium_permeability_dto: PorousMediumPermeabilityDto
        :return: A new instance of PorousMediumPermeability.
        :rtype: PorousMediumPermeability
        """
        return PorousMediumPermeability(porous_medium_permeability_dto.value, porous_medium_permeability_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of PorousMediumPermeability from a DTO unit json representation.

        :param data: The PorousMediumPermeability DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "SquareMeter"}
        :return: A new instance of PorousMediumPermeability.
        :rtype: PorousMediumPermeability
        """
        return PorousMediumPermeability.from_dto(PorousMediumPermeabilityDto.from_json(data))

    def __convert_from_base(self, from_unit: PorousMediumPermeabilityUnits) -> float:
        value = self._value
        
        if from_unit == PorousMediumPermeabilityUnits.Darcy:
            return (value / 9.869233e-13)
        
        if from_unit == PorousMediumPermeabilityUnits.SquareMeter:
            return (value)
        
        if from_unit == PorousMediumPermeabilityUnits.SquareCentimeter:
            return (value / 1e-4)
        
        if from_unit == PorousMediumPermeabilityUnits.Microdarcy:
            return ((value / 9.869233e-13) / 1e-06)
        
        if from_unit == PorousMediumPermeabilityUnits.Millidarcy:
            return ((value / 9.869233e-13) / 0.001)
        
        return None


    def __convert_to_base(self, value: float, to_unit: PorousMediumPermeabilityUnits) -> float:
        
        if to_unit == PorousMediumPermeabilityUnits.Darcy:
            return (value * 9.869233e-13)
        
        if to_unit == PorousMediumPermeabilityUnits.SquareMeter:
            return (value)
        
        if to_unit == PorousMediumPermeabilityUnits.SquareCentimeter:
            return (value * 1e-4)
        
        if to_unit == PorousMediumPermeabilityUnits.Microdarcy:
            return ((value * 9.869233e-13) * 1e-06)
        
        if to_unit == PorousMediumPermeabilityUnits.Millidarcy:
            return ((value * 9.869233e-13) * 0.001)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_darcys(darcys: float):
        """
        Create a new instance of PorousMediumPermeability from a value in darcys.

        

        :param meters: The PorousMediumPermeability value in darcys.
        :type darcys: float
        :return: A new instance of PorousMediumPermeability.
        :rtype: PorousMediumPermeability
        """
        return PorousMediumPermeability(darcys, PorousMediumPermeabilityUnits.Darcy)

    
    @staticmethod
    def from_square_meters(square_meters: float):
        """
        Create a new instance of PorousMediumPermeability from a value in square_meters.

        

        :param meters: The PorousMediumPermeability value in square_meters.
        :type square_meters: float
        :return: A new instance of PorousMediumPermeability.
        :rtype: PorousMediumPermeability
        """
        return PorousMediumPermeability(square_meters, PorousMediumPermeabilityUnits.SquareMeter)

    
    @staticmethod
    def from_square_centimeters(square_centimeters: float):
        """
        Create a new instance of PorousMediumPermeability from a value in square_centimeters.

        

        :param meters: The PorousMediumPermeability value in square_centimeters.
        :type square_centimeters: float
        :return: A new instance of PorousMediumPermeability.
        :rtype: PorousMediumPermeability
        """
        return PorousMediumPermeability(square_centimeters, PorousMediumPermeabilityUnits.SquareCentimeter)

    
    @staticmethod
    def from_microdarcys(microdarcys: float):
        """
        Create a new instance of PorousMediumPermeability from a value in microdarcys.

        

        :param meters: The PorousMediumPermeability value in microdarcys.
        :type microdarcys: float
        :return: A new instance of PorousMediumPermeability.
        :rtype: PorousMediumPermeability
        """
        return PorousMediumPermeability(microdarcys, PorousMediumPermeabilityUnits.Microdarcy)

    
    @staticmethod
    def from_millidarcys(millidarcys: float):
        """
        Create a new instance of PorousMediumPermeability from a value in millidarcys.

        

        :param meters: The PorousMediumPermeability value in millidarcys.
        :type millidarcys: float
        :return: A new instance of PorousMediumPermeability.
        :rtype: PorousMediumPermeability
        """
        return PorousMediumPermeability(millidarcys, PorousMediumPermeabilityUnits.Millidarcy)

    
    @property
    def darcys(self) -> float:
        """
        
        """
        if self.__darcys != None:
            return self.__darcys
        self.__darcys = self.__convert_from_base(PorousMediumPermeabilityUnits.Darcy)
        return self.__darcys

    
    @property
    def square_meters(self) -> float:
        """
        
        """
        if self.__square_meters != None:
            return self.__square_meters
        self.__square_meters = self.__convert_from_base(PorousMediumPermeabilityUnits.SquareMeter)
        return self.__square_meters

    
    @property
    def square_centimeters(self) -> float:
        """
        
        """
        if self.__square_centimeters != None:
            return self.__square_centimeters
        self.__square_centimeters = self.__convert_from_base(PorousMediumPermeabilityUnits.SquareCentimeter)
        return self.__square_centimeters

    
    @property
    def microdarcys(self) -> float:
        """
        
        """
        if self.__microdarcys != None:
            return self.__microdarcys
        self.__microdarcys = self.__convert_from_base(PorousMediumPermeabilityUnits.Microdarcy)
        return self.__microdarcys

    
    @property
    def millidarcys(self) -> float:
        """
        
        """
        if self.__millidarcys != None:
            return self.__millidarcys
        self.__millidarcys = self.__convert_from_base(PorousMediumPermeabilityUnits.Millidarcy)
        return self.__millidarcys

    
    def to_string(self, unit: PorousMediumPermeabilityUnits = PorousMediumPermeabilityUnits.SquareMeter, fractional_digits: int = None) -> str:
        """
        Format the PorousMediumPermeability to a string.
        
        Note: the default format for PorousMediumPermeability is SquareMeter.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the PorousMediumPermeability. Default is 'SquareMeter'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == PorousMediumPermeabilityUnits.Darcy:
            return f"""{super()._truncate_fraction_digits(self.darcys, fractional_digits)} D"""
        
        if unit == PorousMediumPermeabilityUnits.SquareMeter:
            return f"""{super()._truncate_fraction_digits(self.square_meters, fractional_digits)} m²"""
        
        if unit == PorousMediumPermeabilityUnits.SquareCentimeter:
            return f"""{super()._truncate_fraction_digits(self.square_centimeters, fractional_digits)} cm²"""
        
        if unit == PorousMediumPermeabilityUnits.Microdarcy:
            return f"""{super()._truncate_fraction_digits(self.microdarcys, fractional_digits)} μD"""
        
        if unit == PorousMediumPermeabilityUnits.Millidarcy:
            return f"""{super()._truncate_fraction_digits(self.millidarcys, fractional_digits)} mD"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: PorousMediumPermeabilityUnits = PorousMediumPermeabilityUnits.SquareMeter) -> str:
        """
        Get PorousMediumPermeability unit abbreviation.
        Note! the default abbreviation for PorousMediumPermeability is SquareMeter.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == PorousMediumPermeabilityUnits.Darcy:
            return """D"""
        
        if unit_abbreviation == PorousMediumPermeabilityUnits.SquareMeter:
            return """m²"""
        
        if unit_abbreviation == PorousMediumPermeabilityUnits.SquareCentimeter:
            return """cm²"""
        
        if unit_abbreviation == PorousMediumPermeabilityUnits.Microdarcy:
            return """μD"""
        
        if unit_abbreviation == PorousMediumPermeabilityUnits.Millidarcy:
            return """mD"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class ElectricInductanceUnits(Enum):
        """
            ElectricInductanceUnits enumeration
        """
        
        Henry = 'Henry'
        """
            
        """
        
        Picohenry = 'Picohenry'
        """
            
        """
        
        Nanohenry = 'Nanohenry'
        """
            
        """
        
        Microhenry = 'Microhenry'
        """
            
        """
        
        Millihenry = 'Millihenry'
        """
            
        """
        

class ElectricInductanceDto:
    """
    A DTO representation of a ElectricInductance

    Attributes:
        value (float): The value of the ElectricInductance.
        unit (ElectricInductanceUnits): The specific unit that the ElectricInductance value is representing.
    """

    def __init__(self, value: float, unit: ElectricInductanceUnits):
        """
        Create a new DTO representation of a ElectricInductance

        Parameters:
            value (float): The value of the ElectricInductance.
            unit (ElectricInductanceUnits): The specific unit that the ElectricInductance value is representing.
        """
        self.value: float = value
        """
        The value of the ElectricInductance
        """
        self.unit: ElectricInductanceUnits = unit
        """
        The specific unit that the ElectricInductance value is representing
        """

    def to_json(self):
        """
        Get a ElectricInductance DTO JSON object representing the current unit.

        :return: JSON object represents ElectricInductance DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "Henry"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of ElectricInductance DTO from a json representation.

        :param data: The ElectricInductance DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "Henry"}
        :return: A new instance of ElectricInductanceDto.
        :rtype: ElectricInductanceDto
        """
        return ElectricInductanceDto(value=data["value"], unit=ElectricInductanceUnits(data["unit"]))


class ElectricInductance(AbstractMeasure):
    """
    Inductance is a property of an electrical conductor which opposes a change in current.

    Args:
        value (float): The value.
        from_unit (ElectricInductanceUnits): The ElectricInductance unit to create from, The default unit is Henry
    """
    def __init__(self, value: float, from_unit: ElectricInductanceUnits = ElectricInductanceUnits.Henry):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__henries = None
        
        self.__picohenries = None
        
        self.__nanohenries = None
        
        self.__microhenries = None
        
        self.__millihenries = None
        

    def convert(self, unit: ElectricInductanceUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: ElectricInductanceUnits = ElectricInductanceUnits.Henry) -> ElectricInductanceDto:
        """
        Get a new instance of ElectricInductance DTO representing the current unit.

        :param hold_in_unit: The specific ElectricInductance unit to store the ElectricInductance value in the DTO representation.
        :type hold_in_unit: ElectricInductanceUnits
        :return: A new instance of ElectricInductanceDto.
        :rtype: ElectricInductanceDto
        """
        return ElectricInductanceDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: ElectricInductanceUnits = ElectricInductanceUnits.Henry):
        """
        Get a ElectricInductance DTO JSON object representing the current unit.

        :param hold_in_unit: The specific ElectricInductance unit to store the ElectricInductance value in the DTO representation.
        :type hold_in_unit: ElectricInductanceUnits
        :return: JSON object represents ElectricInductance DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "Henry"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(electric_inductance_dto: ElectricInductanceDto):
        """
        Obtain a new instance of ElectricInductance from a DTO unit object.

        :param electric_inductance_dto: The ElectricInductance DTO representation.
        :type electric_inductance_dto: ElectricInductanceDto
        :return: A new instance of ElectricInductance.
        :rtype: ElectricInductance
        """
        return ElectricInductance(electric_inductance_dto.value, electric_inductance_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of ElectricInductance from a DTO unit json representation.

        :param data: The ElectricInductance DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "Henry"}
        :return: A new instance of ElectricInductance.
        :rtype: ElectricInductance
        """
        return ElectricInductance.from_dto(ElectricInductanceDto.from_json(data))

    def __convert_from_base(self, from_unit: ElectricInductanceUnits) -> float:
        value = self._value
        
        if from_unit == ElectricInductanceUnits.Henry:
            return (value)
        
        if from_unit == ElectricInductanceUnits.Picohenry:
            return ((value) / 1e-12)
        
        if from_unit == ElectricInductanceUnits.Nanohenry:
            return ((value) / 1e-09)
        
        if from_unit == ElectricInductanceUnits.Microhenry:
            return ((value) / 1e-06)
        
        if from_unit == ElectricInductanceUnits.Millihenry:
            return ((value) / 0.001)
        
        return None


    def __convert_to_base(self, value: float, to_unit: ElectricInductanceUnits) -> float:
        
        if to_unit == ElectricInductanceUnits.Henry:
            return (value)
        
        if to_unit == ElectricInductanceUnits.Picohenry:
            return ((value) * 1e-12)
        
        if to_unit == ElectricInductanceUnits.Nanohenry:
            return ((value) * 1e-09)
        
        if to_unit == ElectricInductanceUnits.Microhenry:
            return ((value) * 1e-06)
        
        if to_unit == ElectricInductanceUnits.Millihenry:
            return ((value) * 0.001)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_henries(henries: float):
        """
        Create a new instance of ElectricInductance from a value in henries.

        

        :param meters: The ElectricInductance value in henries.
        :type henries: float
        :return: A new instance of ElectricInductance.
        :rtype: ElectricInductance
        """
        return ElectricInductance(henries, ElectricInductanceUnits.Henry)

    
    @staticmethod
    def from_picohenries(picohenries: float):
        """
        Create a new instance of ElectricInductance from a value in picohenries.

        

        :param meters: The ElectricInductance value in picohenries.
        :type picohenries: float
        :return: A new instance of ElectricInductance.
        :rtype: ElectricInductance
        """
        return ElectricInductance(picohenries, ElectricInductanceUnits.Picohenry)

    
    @staticmethod
    def from_nanohenries(nanohenries: float):
        """
        Create a new instance of ElectricInductance from a value in nanohenries.

        

        :param meters: The ElectricInductance value in nanohenries.
        :type nanohenries: float
        :return: A new instance of ElectricInductance.
        :rtype: ElectricInductance
        """
        return ElectricInductance(nanohenries, ElectricInductanceUnits.Nanohenry)

    
    @staticmethod
    def from_microhenries(microhenries: float):
        """
        Create a new instance of ElectricInductance from a value in microhenries.

        

        :param meters: The ElectricInductance value in microhenries.
        :type microhenries: float
        :return: A new instance of ElectricInductance.
        :rtype: ElectricInductance
        """
        return ElectricInductance(microhenries, ElectricInductanceUnits.Microhenry)

    
    @staticmethod
    def from_millihenries(millihenries: float):
        """
        Create a new instance of ElectricInductance from a value in millihenries.

        

        :param meters: The ElectricInductance value in millihenries.
        :type millihenries: float
        :return: A new instance of ElectricInductance.
        :rtype: ElectricInductance
        """
        return ElectricInductance(millihenries, ElectricInductanceUnits.Millihenry)

    
    @property
    def henries(self) -> float:
        """
        
        """
        if self.__henries != None:
            return self.__henries
        self.__henries = self.__convert_from_base(ElectricInductanceUnits.Henry)
        return self.__henries

    
    @property
    def picohenries(self) -> float:
        """
        
        """
        if self.__picohenries != None:
            return self.__picohenries
        self.__picohenries = self.__convert_from_base(ElectricInductanceUnits.Picohenry)
        return self.__picohenries

    
    @property
    def nanohenries(self) -> float:
        """
        
        """
        if self.__nanohenries != None:
            return self.__nanohenries
        self.__nanohenries = self.__convert_from_base(ElectricInductanceUnits.Nanohenry)
        return self.__nanohenries

    
    @property
    def microhenries(self) -> float:
        """
        
        """
        if self.__microhenries != None:
            return self.__microhenries
        self.__microhenries = self.__convert_from_base(ElectricInductanceUnits.Microhenry)
        return self.__microhenries

    
    @property
    def millihenries(self) -> float:
        """
        
        """
        if self.__millihenries != None:
            return self.__millihenries
        self.__millihenries = self.__convert_from_base(ElectricInductanceUnits.Millihenry)
        return self.__millihenries

    
    def to_string(self, unit: ElectricInductanceUnits = ElectricInductanceUnits.Henry, fractional_digits: int = None) -> str:
        """
        Format the ElectricInductance to a string.
        
        Note: the default format for ElectricInductance is Henry.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the ElectricInductance. Default is 'Henry'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == ElectricInductanceUnits.Henry:
            return f"""{super()._truncate_fraction_digits(self.henries, fractional_digits)} H"""
        
        if unit == ElectricInductanceUnits.Picohenry:
            return f"""{super()._truncate_fraction_digits(self.picohenries, fractional_digits)} pH"""
        
        if unit == ElectricInductanceUnits.Nanohenry:
            return f"""{super()._truncate_fraction_digits(self.nanohenries, fractional_digits)} nH"""
        
        if unit == ElectricInductanceUnits.Microhenry:
            return f"""{super()._truncate_fraction_digits(self.microhenries, fractional_digits)} μH"""
        
        if unit == ElectricInductanceUnits.Millihenry:
            return f"""{super()._truncate_fraction_digits(self.millihenries, fractional_digits)} mH"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: ElectricInductanceUnits = ElectricInductanceUnits.Henry) -> str:
        """
        Get ElectricInductance unit abbreviation.
        Note! the default abbreviation for ElectricInductance is Henry.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == ElectricInductanceUnits.Henry:
            return """H"""
        
        if unit_abbreviation == ElectricInductanceUnits.Picohenry:
            return """pH"""
        
        if unit_abbreviation == ElectricInductanceUnits.Nanohenry:
            return """nH"""
        
        if unit_abbreviation == ElectricInductanceUnits.Microhenry:
            return """μH"""
        
        if unit_abbreviation == ElectricInductanceUnits.Millihenry:
            return """mH"""
        